@@ -1,8 +1,8 @@
-extern crate futures;
+use futures;
 use futures::future::Future;
 use futures::stream::Stream;
-extern crate clap;
-extern crate sequoia_ipc as ipc;
+use clap;
+use sequoia_ipc as ipc;
 use crate::ipc::assuan::Client;
 
 fn main() {
