@@ -10,9 +10,10 @@ use std::ptr;
 use std::slice;
 use libc::{c_char, c_int, size_t, time_t};
 
-extern crate sequoia_openpgp as openpgp;
+use sequoia_openpgp as openpgp;
 use self::openpgp::{
     crypto,
+    crypto::Password,
     types::ReasonForRevocation,
     parse::{
         PacketParserResult,
@@ -108,7 +109,7 @@ fn pgp_cert_merge(errp: Option<&mut *mut crate::error::Error>,
 /// Consumes `cert` and the packets in `packets`.  The buffer, however,
 /// must be managed by the caller.
 #[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
-fn pgp_cert_merge_packets(errp: Option<&mut *mut crate::error::Error>,
+fn pgp_cert_insert_packets(errp: Option<&mut *mut crate::error::Error>,
                          cert: *mut Cert,
                          packets: *mut *mut Packet,
                          packets_len: size_t)
@@ -119,7 +120,7 @@ fn pgp_cert_merge_packets(errp: Option<&mut *mut crate::error::Error>,
     };
     let packets =
         packets.iter_mut().map(|&mut p| p.move_from_raw());
-    cert.merge_packets(packets).move_into_raw(errp)
+    cert.insert_packets(packets).move_into_raw(errp)
 }
 
 /// Returns the fingerprint.
@@ -185,7 +186,7 @@ fn int_to_reason_for_revocation(code: c_int) -> ReasonForRevocation {
 ///
 /// This function does *not* consume `cert`.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```c
 /// #include <assert.h>
@@ -216,10 +217,9 @@ fn int_to_reason_for_revocation(code: c_int) -> ReasonForRevocation {
 ///                              "It was the maid :/");
 /// assert (revocation);
 /// pgp_signer_free (primary_signer);
-/// pgp_key_pair_free (primary_keypair);
 ///
 /// pgp_packet_t packet = pgp_signature_into_packet (revocation);
-/// cert = pgp_cert_merge_packets (NULL, cert, &packet, 1);
+/// cert = pgp_cert_insert_packets (NULL, cert, &packet, 1);
 /// assert (cert);
 ///
 /// pgp_revocation_status_t rs = pgp_cert_revocation_status (cert, policy, 0);
@@ -253,6 +253,33 @@ fn pgp_cert_revoke(errp: Option<&mut *mut crate::error::Error>,
     sig.move_into_raw(errp)
 }
 
+/// Returns a new revocation certificate for the Cert.
+///
+/// This function consumes `cert` and returns a new `Cert`.
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle] pub extern "C"
+fn pgp_cert_revoke_in_place(errp: Option<&mut *mut crate::error::Error>,
+                            cert: *mut Cert,
+                            primary_signer: *mut Box<dyn crypto::Signer>,
+                            code: c_int,
+                            reason: Option<&c_char>)
+                  -> Maybe<Cert>
+{
+    ffi_make_fry_from_errp!(errp);
+    let cert = cert.move_from_raw();
+    let signer = ffi_param_ref_mut!(primary_signer);
+    let code = int_to_reason_for_revocation(code);
+    let reason = if let Some(reason) = reason {
+        ffi_param_cstr!(reason as *const c_char).to_bytes()
+    } else {
+        b""
+    };
+
+    let builder = CertRevocationBuilder::new();
+    let builder = ffi_try_or!(builder.set_reason_for_revocation(code, reason), None);
+    let sig = builder.build(signer.as_mut(), &cert, None);
+    cert.insert_packets(sig).move_into_raw(errp)
+}
+
 /// Returns whether the Cert is alive at the specified time.
 ///
 /// If `when` is 0, then the current time is used.
@@ -286,7 +313,7 @@ fn pgp_cert_set_expiration_time(errp: Option<&mut *mut crate::error::Error>,
 
     let sigs = ffi_try_or!(cert.set_expiration_time(policy, None, signer.as_mut(),
                                                     maybe_time(expiry)), None);
-    cert.merge_packets(sigs).move_into_raw(errp)
+    cert.insert_packets(sigs).move_into_raw(errp)
 }
 
 /// Returns whether the Cert includes any secret key material.
@@ -348,7 +375,7 @@ pub extern "C" fn pgp_cert_user_id_iter_policy<'a>(
     -> *mut ValidUserIDIterWrapper<'static>
 {
     let policy = policy.ref_raw();
-    let iter_wrapper = ffi_param_ref_mut!(iter_wrapper);
+    let mut iter_wrapper = ffi_param_move!(iter_wrapper);
     if iter_wrapper.next_called {
         panic!("Can't change UserIDIter filter after iterating.");
     }
@@ -519,7 +546,7 @@ pub extern "C" fn pgp_cert_key_iter_policy<'a>(
     -> *mut ValidKeyAmalgamationIterWrapper<'static>
 {
     let policy = policy.ref_raw();
-    let iter_wrapper = ffi_param_ref_mut!(iter_wrapper);
+    let mut iter_wrapper = ffi_param_move!(iter_wrapper);
     if iter_wrapper.next_called {
         panic!("Can't change KeyAmalgamationIter filter after iterating.");
     }
@@ -854,7 +881,7 @@ pub extern "C" fn pgp_cert_parser_free(parser: Option<&mut CertParserWrapper>)
 
 /// Creates a default `pgp_cert_builder_t`.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```c
 /// #include <assert.h>
@@ -936,6 +963,22 @@ pub extern "C" fn pgp_cert_builder_set_cipher_suite
     *certb = box_raw!(certb_);
 }
 
+/// Sets the password for primary and all subkeys.
+#[::sequoia_ffi_macros::extern_fn] #[no_mangle]
+pub extern "C" fn pgp_cert_builder_set_password
+    (certb: *mut *mut CertBuilder, password: *const u8, password_len: size_t)
+{
+    let certb = ffi_param_ref_mut!(certb);
+    let certb_ = ffi_param_move!(*certb);
+    assert!(!password.is_null());
+    let password = unsafe {
+        slice::from_raw_parts(password, password_len as usize)
+    };
+    let password: Password = password.into();
+    let certb_ = certb_.set_password(Some(password));
+    *certb = box_raw!(certb_);
+}
+
 /// Adds a new user ID. The first user ID added replaces the default
 /// ID that is just the empty string.
 ///
