@@ -315,16 +315,11 @@
 
 #![warn(missing_docs)]
 
-#[macro_use]
-extern crate lazy_static;
-extern crate libc;
-extern crate memsec;
+use libc;
 
-extern crate sequoia_ffi_macros;
+use sequoia_ffi_macros;
 use sequoia_ffi_macros::{
     ffi_wrapper_type,
 };
 
-extern crate sequoia_openpgp;
-
 include!("common.rs");
