@@ -4,8 +4,11 @@
 use std::env;
 use std::io;
 
-extern crate sequoia_openpgp as openpgp;
-use crate::openpgp::armor;
+use anyhow::Context;
+
+use sequoia_openpgp as openpgp;
+
+use crate::openpgp::serialize::stream::Armorer;
 use crate::openpgp::types::KeyFlags;
 use crate::openpgp::parse::Parse;
 use crate::openpgp::serialize::stream::{
@@ -13,29 +16,29 @@ use crate::openpgp::serialize::stream::{
 };
 use crate::openpgp::policy::StandardPolicy as P;
 
-fn main() {
+fn main() -> openpgp::Result<()> {
     let p = &P::new();
 
     let args: Vec<String> = env::args().collect();
     if args.len() < 3 {
-        panic!("A simple encryption filter.\n\n\
+        return Err(anyhow::anyhow!("A simple encryption filter.\n\n\
                 Usage: {} [at-rest|for-transport] <keyfile> [<keyfile>...] \
-                <input >output\n", args[0]);
+                <input >output\n", args[0]));
     }
 
     let mode = match args[1].as_ref() {
-        "at-rest" => KeyFlags::default().set_storage_encryption(true),
-        "for-transport" => KeyFlags::default().set_transport_encryption(true),
-        x => panic!("invalid mode: {:?}, \
+        "at-rest" => KeyFlags::empty().set_storage_encryption(),
+        "for-transport" => KeyFlags::empty().set_transport_encryption(),
+        x => return Err(anyhow::anyhow!("invalid mode: {:?}, \
                      must be either 'at-rest' or 'for-transport'",
-                    x),
+                    x)),
     };
 
     // Read the certificates from the given files.
     let certs: Vec<openpgp::Cert> = args[2..].iter().map(|f| {
         openpgp::Cert::from_file(f)
-            .expect("Failed to read key")
-    }).collect();
+    }).collect::<openpgp::Result<Vec<_>>>()
+    .context("Failed to read key")?;
 
     let recipients =
         certs.iter()
@@ -45,30 +48,28 @@ fn main() {
         });
 
     // Compose a writer stack corresponding to the output format and
-    // packet structure we want.  First, we want the output to be
-    // ASCII armored.
-    let mut sink = armor::Writer::new(io::stdout(), armor::Kind::Message)
-        .expect("Failed to create an armored writer");
+    // packet structure we want.
+    let mut sink = io::stdout();
 
     // Stream an OpenPGP message.
     let message = Message::new(&mut sink);
 
-    // We want to encrypt a literal data packet.
-    let encryptor = Encryptor::for_recipients(message, recipients)
-        .build().expect("Failed to create encryptor");
+    let message = Armorer::new(message).build()?;
 
-    let mut literal_writer = LiteralWriter::new(encryptor).build()
-        .expect("Failed to create literal writer");
+    // We want to encrypt a literal data packet.
+    let message = Encryptor::for_recipients(message, recipients)
+        .build().context("Failed to create encryptor")?;
+
+    let mut message = LiteralWriter::new(message).build()
+        .context("Failed to create literal writer")?;
 
     // Copy stdin to our writer stack to encrypt the data.
-    io::copy(&mut io::stdin(), &mut literal_writer)
-        .expect("Failed to encrypt");
+    io::copy(&mut io::stdin(), &mut message)
+        .context("Failed to encrypt")?;
 
     // Finally, finalize the OpenPGP message by tearing down the
     // writer stack.
-    literal_writer.finalize().unwrap();
+    message.finalize()?;
 
-    // Finalize the armor writer.
-    sink.finalize()
-        .expect("Failed to write data");
+    Ok(())
 }
