@@ -16,6 +16,8 @@ use crate::{
 
 /// An iterator over `Key`s.
 ///
+/// An iterator over [`KeyAmalgamation`]s.
+///
 /// A `KeyAmalgamationIter` is like a [`ComponentAmalgamationIter`],
 /// but specialized for keys.  Refer to the [module documentation] for
 /// an explanation of why a different type is necessary.
@@ -42,6 +44,7 @@ use crate::{
 ///
 /// A `KeyAmalgamationIter` is returned by [`Cert::keys`].
 ///
+/// [`KeyAmalgamation`]: struct.KeyAmalgamation.html
 /// [`ComponentAmalgamationIter`]: ../struct.ComponentAmalgamationIter.html
 /// [module documentation]: index.html
 /// [`KeyAmalgamationIter::with_policy`]: ../trait.ValidateAmalgamation.html
@@ -70,7 +73,7 @@ pub struct KeyAmalgamationIter<'a, P, R>
     unencrypted_secret: Option<bool>,
 
     // Only return keys in this set.
-    key_handles: Vec<KeyHandle>,
+    key_handles: Option<Vec<KeyHandle>>,
 
     _p: std::marker::PhantomData<P>,
     _r: std::marker::PhantomData<R>,
@@ -152,8 +155,8 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 
             t!("Considering key: {:?}", ka.key());
 
-            if self.key_handles.len() > 0 {
-                if !self.key_handles
+            if let Some(key_handles) = self.key_handles.as_ref() {
+                if !key_handles
                     .iter()
                     .any(|h| h.aliases(ka.key().key_handle()))
                 {
@@ -217,7 +220,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             // The filters.
             secret: None,
             unencrypted_secret: None,
-            key_handles: Vec::with_capacity(0),
+            key_handles: None,
 
             _p: std::marker::PhantomData,
             _r: std::marker::PhantomData,
@@ -230,7 +233,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// # fn main() { f().unwrap(); }
@@ -266,7 +269,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// # fn main() { f().unwrap(); }
@@ -309,7 +312,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// # fn main() { f().unwrap(); }
@@ -334,7 +337,10 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
-        self.key_handles.push(h.into());
+        if self.key_handles.is_none() {
+            self.key_handles = Some(Vec::new());
+        }
+        self.key_handles.as_mut().unwrap().push(h.into());
         self
     }
 
@@ -351,7 +357,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// # fn main() { f().unwrap(); }
@@ -359,7 +365,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// #     let (cert, _) =
     /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handles = &[ cert.primary_key().key_handle() ][..];
+    /// # let key_handles = &[cert.primary_key().key_handle()][..];
     /// # let mut i = 0;
     /// for ka in cert.keys().key_handles(key_handles.iter()) {
     ///     // Use it.
@@ -377,7 +383,10 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
         -> Self
         where 'a: 'b
     {
-        self.key_handles.extend(h.map(|h| h.clone()));
+        if self.key_handles.is_none() {
+            self.key_handles = Some(Vec::new());
+        }
+        self.key_handles.as_mut().unwrap().extend(h.cloned());
         self
     }
 
@@ -390,7 +399,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// #
@@ -445,7 +454,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     /// #
@@ -496,12 +505,14 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 
 /// An iterator over valid `Key`s.
 ///
-/// A `ValidComponentAmalgamationIter` is a [`KeyAmalgamationIter`]
+/// An iterator over [`ValidKeyAmalgamation`]s.
+///
+/// A `ValidKeyAmalgamationIter` is a [`KeyAmalgamationIter`]
 /// that includes a [`Policy`] and a reference time, which it firstly
 /// uses to only return valid `Key`s.  (For a definition of valid
 /// keys, see the documentation for [`ValidateAmalgamation`].)
 ///
-/// A `ValidComponentAmalgamationIter` also provides additional
+/// A `ValidKeyAmalgamationIter` also provides additional
 /// filters based on information available in the `Key`s' binding
 /// signatures.  For instance, [`ValidKeyAmalgamationIter::revoked`]
 /// filters the returned `Key`s by whether or not they are revoked.
@@ -520,7 +531,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 /// Find a key that we can use to sign a document:
 ///
 /// ```
-/// # extern crate sequoia_openpgp as openpgp;
+/// # use sequoia_openpgp as openpgp;
 /// # use openpgp::cert::prelude::*;
 /// use openpgp::policy::StandardPolicy;
 /// use openpgp::types::RevocationStatus;
@@ -564,6 +575,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 /// # }
 /// ```
 ///
+/// [`ValidKeyAmalgamation`]: struct.ValidKeyAmalgamation.html
 /// [`KeyAmalgamationIter`]: struct.KeyAmalgamationIter.html
 /// [`Policy`]: ../../../policy/trait.Policy.html
 /// [`ValidateAmalgamation`]: ../trait.ValidateAmalgamation.html
@@ -595,7 +607,7 @@ pub struct ValidKeyAmalgamationIter<'a, P, R>
     unencrypted_secret: Option<bool>,
 
     // Only return keys in this set.
-    key_handles: Vec<KeyHandle>,
+    key_handles: Option<Vec<KeyHandle>>,
 
     // If not None, only returns keys with the specified flags.
     flags: Option<KeyFlags>,
@@ -719,8 +731,8 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
             let key = ka.key();
             t!("Considering key: {:?}", key);
 
-            if self.key_handles.len() > 0 {
-                if !self.key_handles
+            if let Some(key_handles) = self.key_handles.as_ref() {
+                if !key_handles
                     .iter()
                     .any(|h| h.aliases(key.key_handle()))
                 {
@@ -834,7 +846,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     /// use openpgp::types::KeyFlags;
@@ -853,8 +865,8 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// for ka in cert.keys()
     ///     .with_policy(p, None)
     ///     .key_flags(KeyFlags::empty()
-    ///         .set_transport_encryption(true)
-    ///         .set_storage_encryption(true))
+    ///         .set_transport_encryption()
+    ///         .set_storage_encryption())
     /// {
     ///     // Valid encryption-capable keys.
     /// #   i += 1;
@@ -905,7 +917,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -935,7 +947,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///   [Section 12.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.2.3.21
     ///   [`ValidKeyAmalgamation::key_flags`]: struct.ValidKeyAmalgamation.html#method.key_flags
     pub fn for_certification(self) -> Self {
-        self.key_flags(KeyFlags::default().set_certification(true))
+        self.key_flags(KeyFlags::empty().set_certification())
     }
 
     /// Returns signing-capable keys.
@@ -950,7 +962,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -978,7 +990,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///
     ///   [`ValidKeyAmalgamation::for_signing`]: struct.ValidKeyAmalgamation.html#method.for_signing
     pub fn for_signing(self) -> Self {
-        self.key_flags(KeyFlags::default().set_signing(true))
+        self.key_flags(KeyFlags::empty().set_signing())
     }
 
     /// Returns authentication-capable keys.
@@ -993,7 +1005,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1021,7 +1033,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///
     ///   [`ValidKeyAmalgamation::for_authentication`]: struct.ValidKeyAmalgamation.html#method.for_authentication
     pub fn for_authentication(self) -> Self {
-        self.key_flags(KeyFlags::default().set_authentication(true))
+        self.key_flags(KeyFlags::empty().set_authentication())
     }
 
     /// Returns encryption-capable keys for data at rest.
@@ -1036,7 +1048,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1064,7 +1076,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///
     ///   [`ValidKeyAmalgamation::for_storage_encryption`]: struct.ValidKeyAmalgamation.html#method.for_storage_encryption
     pub fn for_storage_encryption(self) -> Self {
-        self.key_flags(KeyFlags::default().set_storage_encryption(true))
+        self.key_flags(KeyFlags::empty().set_storage_encryption())
     }
 
     /// Returns encryption-capable keys for data in transit.
@@ -1079,7 +1091,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1107,7 +1119,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     ///
     ///   [`ValidKeyAmalgamation::for_transport_encryption`]: struct.ValidKeyAmalgamation.html#method.for_transport_encryption
     pub fn for_transport_encryption(self) -> Self {
-        self.key_flags(KeyFlags::default().set_transport_encryption(true))
+        self.key_flags(KeyFlags::empty().set_transport_encryption())
     }
 
     /// Returns keys that are alive.
@@ -1122,7 +1134,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1169,7 +1181,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1198,7 +1210,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// `ValidKeyAmalgamationIter::revoked(false)` is equivalent to:
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// use openpgp::types::RevocationStatus;
     /// # use openpgp::cert::prelude::*;
@@ -1257,7 +1269,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
@@ -1303,7 +1315,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
@@ -1356,7 +1368,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
@@ -1385,7 +1397,10 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
-        self.key_handles.push(h.into());
+        if self.key_handles.is_none() {
+            self.key_handles = Some(Vec::new());
+        }
+        self.key_handles.as_mut().unwrap().push(h.into());
         self
     }
 
@@ -1402,7 +1417,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Result;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
@@ -1414,7 +1429,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// #     let (cert, _) =
     /// #         CertBuilder::general_purpose(None, Some("alice@example.org"))
     /// #         .generate()?;
-    /// # let key_handles = &[ cert.primary_key().key_handle() ][..];
+    /// # let key_handles = &[cert.primary_key().key_handle()][..];
     /// # let mut i = 0;
     /// for ka in cert.keys().with_policy(p, None).key_handles(key_handles.iter()) {
     ///     // Use it.
@@ -1432,7 +1447,10 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
         -> Self
         where 'a: 'b
     {
-        self.key_handles.extend(h.map(|h| h.clone()));
+        if self.key_handles.is_none() {
+            self.key_handles = Some(Vec::new());
+        }
+        self.key_handles.as_mut().unwrap().extend(h.cloned());
         self
     }
 
@@ -1445,7 +1463,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Examples
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::cert::prelude::*;
     /// use openpgp::policy::StandardPolicy;
     ///
@@ -1515,7 +1533,7 @@ mod test {
         let p = &P::new();
         let (cert, _) = CertBuilder::new()
             .generate().unwrap();
-        let flags = KeyFlags::default().set_transport_encryption(true);
+        let flags = KeyFlags::empty().set_transport_encryption();
 
         assert_eq!(cert.keys().with_policy(p, None).key_flags(flags).count(), 0);
     }
@@ -1526,7 +1544,7 @@ mod test {
         let (cert, _) = CertBuilder::new()
             .add_transport_encryption_subkey()
             .generate().unwrap();
-        let flags = KeyFlags::default().set_transport_encryption(true);
+        let flags = KeyFlags::empty().set_transport_encryption();
 
         assert_eq!(cert.keys().with_policy(p, None).key_flags(flags).count(), 1);
     }
@@ -1538,7 +1556,7 @@ mod test {
             .add_transport_encryption_subkey()
             .add_signing_subkey()
             .generate().unwrap();
-        let flags = KeyFlags::default().set_transport_encryption(true);
+        let flags = KeyFlags::empty().set_transport_encryption();
 
         assert_eq!(cert.keys().with_policy(p, None).key_flags(flags).count(), 1);
     }
@@ -1549,7 +1567,7 @@ mod test {
         let (cert, _) = CertBuilder::new()
             .add_transport_encryption_subkey()
             .generate().unwrap();
-        let flags = KeyFlags::default().set_transport_encryption(true);
+        let flags = KeyFlags::empty().set_transport_encryption();
 
         let now = SystemTime::now()
             - std::time::Duration::new(52 * 7 * 24 * 60 * 60, 0);
@@ -1563,7 +1581,7 @@ mod test {
         let (cert, _) = CertBuilder::new()
             .add_certification_subkey()
             .generate().unwrap();
-        let flags = KeyFlags::default().set_certification(true);
+        let flags = KeyFlags::empty().set_certification();
 
         assert_eq!(cert.keys().with_policy(p, None).key_flags(flags).count(),
                    2);
@@ -1593,7 +1611,7 @@ mod test {
                        .for_signing().count(),
                    1);
         assert_eq!(cert.keys().with_policy(p, None).alive().revoked(false)
-                       .key_flags(KeyFlags::default().set_authentication(true))
+                       .key_flags(KeyFlags::empty().set_authentication())
                        .count(),
                    1);
     }
