@@ -71,7 +71,7 @@ use crate::cert::prelude::*;
 ///     .build(&mut signer, &cert, None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.merge_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?;
 ///
 /// // Now it's revoked.
 /// assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -215,7 +215,7 @@ impl CertRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::KeyRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.merge_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -231,16 +231,7 @@ impl CertRevocationBuilder {
 
         cert.primary_key().hash(&mut hash);
 
-        let creation_time
-            = self.signature_creation_time()
-            .unwrap_or_else(time::SystemTime::now);
-
-        self.builder
-            // If not set, set it to now.
-            .set_signature_creation_time(creation_time)?
-            .set_issuer_fingerprint(signer.public().fingerprint())?
-            .set_issuer(signer.public().keyid())?
-            .sign_hash(signer, hash)
+        self.builder.sign_hash(signer, hash)
     }
 }
 
@@ -308,7 +299,7 @@ impl Deref for CertRevocationBuilder {
 ///     .build(&mut signer, &cert, subkey.key(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.merge_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?;
 ///
 /// // Now it's revoked.
 /// let subkey = cert.keys().subkeys().nth(0).unwrap();
@@ -451,7 +442,7 @@ impl SubkeyRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::SubkeyRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.merge_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -539,7 +530,7 @@ impl Deref for SubkeyRevocationBuilder {
 ///     .build(&mut signer, &cert, ua.userid(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.merge_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?;
 ///
 /// // Now it's revoked.
 /// let ua = cert.userids().nth(0).unwrap();
@@ -689,7 +680,7 @@ impl UserIDRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.merge_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
@@ -784,7 +775,7 @@ impl Deref for UserIDRevocationBuilder {
 ///     .build(&mut signer, &cert, ua.user_attribute(), None)?;
 ///
 /// // Merge it into the certificate.
-/// let cert = cert.merge_packets(sig.clone())?;
+/// let cert = cert.insert_packets(sig.clone())?;
 ///
 /// // Now it's revoked.
 /// let ua = cert.user_attributes().nth(0).unwrap();
@@ -940,7 +931,7 @@ impl UserAttributeRevocationBuilder {
     /// # assert_eq!(sig.typ(), SignatureType::CertificationRevocation);
     /// #
     /// # // Merge it into the certificate.
-    /// # let cert = cert.merge_packets(sig.clone())?;
+    /// # let cert = cert.insert_packets(sig.clone())?;
     /// #
     /// # // Now it's revoked.
     /// # assert_eq!(RevocationStatus::Revoked(vec![&sig]),
