@@ -1,156 +1,49 @@
-//! Elliptic Curve Diffie-Hellman.
+//! Elliptic-curve Diffie-Hellman.
+//!
+//! Sequoia implements the Elliptic-curve Diffie-Hellman key agreement
+//! protocol for use in OpenPGP as described by [RFC 6637].  In short,
+//! a shared secret is derived using Elliptic-curve Diffie-Hellman, a
+//! wrapping key is derived from that shared secret, and the message's
+//! session key is wrapped using that wrapping key.
+//!
+//!   [RFC 6637]: https://tools.ietf.org/html/rfc6637
 
 use crate::vec_truncate;
-use crate::Error;
-use crate::packet::{
-    Key,
-    key,
-};
-use crate::Result;
-use crate::types::{
-    Curve,
-    HashAlgorithm,
-    SymmetricAlgorithm,
-    PublicKeyAlgorithm,
-};
-use crate::utils::{
-    write_be_u64,
-    read_be_u64,
-};
+use crate::{Error, Result};
+
 use crate::crypto::SessionKey;
 use crate::crypto::mem::Protected;
-use crate::crypto::mpi::{MPI, PublicKey, SecretKeyMaterial, Ciphertext};
-use nettle::{cipher, curve25519, mode, mode::Mode, ecc, ecdh, random::Yarrow};
+use crate::crypto::mpi::{self, MPI};
+use crate::key;
+use crate::packet::Key;
+use crate::types::{Curve, HashAlgorithm, PublicKeyAlgorithm, SymmetricAlgorithm};
+use crate::utils::{read_be_u64, write_be_u64};
 
-/// Wraps a session key using Elliptic Curve Diffie-Hellman.
-#[allow(non_snake_case)]
-pub fn encrypt<R>(recipient: &Key<key::PublicParts, R>,
-                  session_key: &SessionKey)
-    -> Result<Ciphertext>
-    where R: key::KeyRole
-{
-    let mut rng = Yarrow::default();
-
-    if let &PublicKey::ECDH {
-        ref curve, ref q,..
-    } = recipient.mpis() {
-        match curve {
-            Curve::Cv25519 => {
-                // Obtain the authenticated recipient public key R
-                let R = q.decode_point(curve)?.0;
-
-                // Generate an ephemeral key pair {v, V=vG}
-                let v: Protected =
-                    curve25519::private_key(&mut rng).into();
-
-                // Compute the public key.  We need to add an encoding
-                // octet in front of the key.
-                let mut VB = [0x40; 1 + curve25519::CURVE25519_SIZE];
-                curve25519::mul_g(&mut VB[1..], &v)
-                    .expect("buffers are of the wrong size");
-                let VB = MPI::new(&VB);
-
-                // Compute the shared point S = vR;
-                let mut S: Protected =
-                    vec![0; curve25519::CURVE25519_SIZE].into();
-                curve25519::mul(&mut S, &v, R)
-                    .expect("buffers are of the wrong size");
-
-                encrypt_shared(recipient, session_key, VB, &S)
-            }
-            Curve::NistP256 | Curve::NistP384 | Curve::NistP521 => {
-                // Obtain the authenticated recipient public key R and
-                // generate an ephemeral private key v.
-
-                // Note: ecc::Point and ecc::Scalar are cleaned up by
-                // nettle.
-                let (Rx, Ry) = q.decode_point(curve)?;
-                let (R, v, field_sz) = match curve {
-                    Curve::NistP256 => {
-                        let R = ecc::Point::new::<ecc::Secp256r1>(Rx, Ry)?;
-                        let v =
-                            ecc::Scalar::new_random::<ecc::Secp256r1, _>(&mut rng);
-                        let field_sz = 256;
-
-                        (R, v, field_sz)
-                    }
-                    Curve::NistP384 => {
-                        let R = ecc::Point::new::<ecc::Secp384r1>(Rx, Ry)?;
-                        let v =
-                            ecc::Scalar::new_random::<ecc::Secp384r1, _>(&mut rng);
-                        let field_sz = 384;
-
-                        (R, v, field_sz)
-                    }
-                    Curve::NistP521 => {
-                        let R = ecc::Point::new::<ecc::Secp521r1>(Rx, Ry)?;
-                        let v =
-                            ecc::Scalar::new_random::<ecc::Secp521r1, _>(&mut rng);
-                        let field_sz = 521;
-
-                        (R, v, field_sz)
-                    }
-                    _ => unreachable!(),
-                };
-
-                // Compute the public key.
-                let VB = ecdh::point_mul_g(&v);
-                let (VBx, VBy) = VB.as_bytes();
-                let VB = MPI::new_weierstrass(&VBx, &VBy, field_sz);
-
-                // Compute the shared point S = vR;
-                let S = ecdh::point_mul(&v, &R)?;
-
-                // Get the X coordinate, safely dispose of Y.
-                let (Sx, Sy) = S.as_bytes();
-                Protected::from(Sy); // Just a precaution.
-
-                // Zero-pad to the size of the underlying field,
-                // rounded to the next byte.
-                let mut Sx = Vec::from(Sx);
-                while Sx.len() < (field_sz + 7) / 8 {
-                    Sx.insert(0, 0);
-                }
-
-                encrypt_shared(recipient, session_key, VB, &Sx.into())
-            }
-
-            // Not implemented in Nettle
-            Curve::BrainpoolP256 | Curve::BrainpoolP512 =>
-                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
-
-            // N/A
-            Curve::Unknown(_) | Curve::Ed25519 =>
-                Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
-
-            Curve::__Nonexhaustive => unreachable!(),
-        }
-    } else {
-        Err(Error::InvalidArgument("Expected an ECDHPublicKey".into()).into())
-    }
-}
+pub(crate) use crate::crypto::backend::ecdh::{encrypt, decrypt};
 
 /// Wraps a session key.
 ///
-/// After using Elliptic Curve Diffie-Hellman to compute a shared
-/// secret, this function deterministically encrypts the given session
-/// key.
+/// After using Elliptic-curve Diffie-Hellman to compute the shared
+/// secret, this function deterministically derives the wrapping key
+/// from the shared secret, and uses it to wrap (i.e. encrypt) the
+/// given session key.
 ///
-/// `VB` is the ephemeral public key (with 0x40 prefix), `S` is the
-/// shared Diffie-Hellman secret.
+/// `VB` is the ephemeral public key encoded appropriately as MPI
+/// (i.e. with the 0x40 prefix for X25519, or 0x04 for the NIST
+/// curves), `S` is the shared Diffie-Hellman secret.
 #[allow(non_snake_case)]
-pub fn encrypt_shared<R>(recipient: &Key<key::PublicParts, R>,
-                         session_key: &SessionKey, VB: MPI,
-                         S: &Protected)
-    -> Result<Ciphertext>
+pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
+                              session_key: &SessionKey, VB: MPI,
+                              S: &Protected)
+    -> Result<mpi::Ciphertext>
     where R: key::KeyRole
 {
     match recipient.mpis() {
-        &PublicKey::ECDH{ ref curve, ref hash, ref sym,.. } => {
+        &mpi::PublicKey::ECDH { ref curve, ref hash, ref sym,.. } => {
             // m = sym_alg_ID || session key || checksum || pkcs5_padding;
             let mut m = Vec::with_capacity(40);
             m.extend_from_slice(session_key);
-            let m = pkcs5_pad(m.into(), 40);
+            let m = pkcs5_pad(m.into(), 40)?;
             // Note: We always pad up to 40 bytes to obfuscate the
             // length of the symmetric key.
 
@@ -167,7 +60,7 @@ pub fn encrypt_shared<R>(recipient: &Key<key::PublicParts, R>,
             let C = aes_key_wrap(*sym, &Z, &m)?;
 
             // Output (MPI(VB) || len(C) || C).
-            Ok(Ciphertext::ECDH {
+            Ok(mpi::Ciphertext::ECDH {
                 e: VB,
                 key: C.into_boxed_slice(),
             })
@@ -178,131 +71,25 @@ pub fn encrypt_shared<R>(recipient: &Key<key::PublicParts, R>,
     }
 }
 
-/// Unwraps a session key using Elliptic Curve Diffie-Hellman.
-#[allow(non_snake_case)]
-pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
-                  recipient_sec: &SecretKeyMaterial,
-                  ciphertext: &Ciphertext)
-    -> Result<SessionKey>
-    where R: key::KeyRole
-{
-    match (recipient.mpis(), recipient_sec, ciphertext) {
-        (PublicKey::ECDH { ref curve, ..},
-         SecretKeyMaterial::ECDH { ref scalar, },
-         Ciphertext::ECDH { ref e, .. }) =>
-        {
-            let S: Protected = match curve {
-                Curve::Cv25519 => {
-                    // Get the public part V of the ephemeral key.
-                    let V = e.decode_point(curve)?.0;
-
-                    // Nettle expects the private key to be exactly
-                    // CURVE25519_SIZE bytes long but OpenPGP allows leading
-                    // zeros to be stripped.
-                    // Padding has to be unconditional; otherwise we have a
-                    // secret-dependent branch.
-                    //
-                    // Reverse the scalar.  See
-                    // https://lists.gnupg.org/pipermail/gnupg-devel/2018-February/033437.html.
-                    let missing = curve25519::CURVE25519_SIZE
-                        .saturating_sub(scalar.value().len());
-                    let mut r = [0u8; curve25519::CURVE25519_SIZE];
-
-                    r[missing..].copy_from_slice(scalar.value());
-                    r.reverse();
-
-                    // Compute the shared point S = rV = rvG, where (r, R)
-                    // is the recipient's key pair.
-                    let mut S: Protected =
-                        vec![0; curve25519::CURVE25519_SIZE].into();
-                    let res = curve25519::mul(&mut S, &r[..], V);
-
-                    unsafe {
-                        memsec::memzero(r.as_mut_ptr(),
-                        curve25519::CURVE25519_SIZE);
-                    }
-                    res.expect("buffers are of the wrong size");
-                    S
-                }
-
-                Curve::NistP256 | Curve::NistP384 | Curve::NistP521 => {
-                    // Get the public part V of the ephemeral key and
-                    // compute the shared point S = rV = rvG, where (r, R)
-                    // is the recipient's key pair.
-                    let (Vx, Vy) = e.decode_point(curve)?;
-                    let (V, r, field_sz) = match curve {
-                        Curve::NistP256 => {
-                            let V =
-                                ecc::Point::new::<ecc::Secp256r1>(&Vx, &Vy)?;
-                            let r =
-                                ecc::Scalar::new::<ecc::Secp256r1>(scalar.value())?;
-
-                            (V, r, 256)
-                        }
-                        Curve::NistP384 => {
-                            let V =
-                                ecc::Point::new::<ecc::Secp384r1>(&Vx, &Vy)?;
-                            let r =
-                                ecc::Scalar::new::<ecc::Secp384r1>(scalar.value())?;
-
-                            (V, r, 384)
-                        }
-                        Curve::NistP521 => {
-                            let V =
-                                ecc::Point::new::<ecc::Secp521r1>(&Vx, &Vy)?;
-                            let r =
-                                ecc::Scalar::new::<ecc::Secp521r1>(scalar.value())?;
-
-                            (V, r, 521)
-                        }
-                        _ => unreachable!(),
-                    };
-                    let S = ecdh::point_mul(&r, &V)?;
-
-                    // Get the X coordinate, safely dispose of Y.
-                    let (Sx, Sy) = S.as_bytes();
-                    Protected::from(Sy); // Just a precaution.
-
-                    // Zero-pad to the size of the underlying field,
-                    // rounded to the next byte.
-                    let mut Sx = Vec::from(Sx);
-                    while Sx.len() < (field_sz + 7) / 8 {
-                        Sx.insert(0, 0);
-                    }
-
-                    Sx.into()
-                }
-
-                _ => {
-                    return Err(Error::UnsupportedEllipticCurve(curve.clone()).into());
-                }
-            };
-
-            decrypt_shared(recipient, &S, ciphertext)
-        }
-
-        _ =>
-            Err(Error::InvalidArgument("Expected an ECDHPublicKey".into()).into()),
-    }
-}
-
 /// Unwraps a session key.
 ///
-/// After using Elliptic Curve Diffie-Hellman to compute the shared
-/// secret, this function decrypts the given encrypted session key.
+/// After using Elliptic-curve Diffie-Hellman to compute the shared
+/// secret, this function deterministically derives the wrapping key
+/// from the shared secret, and uses it to unwrap (i.e. decrypt) the
+/// session key.
 ///
 /// `recipient` is the message receiver's public key, `S` is the
 /// shared Diffie-Hellman secret used to encrypt `ciphertext`.
 #[allow(non_snake_case)]
-pub fn decrypt_shared<R>(recipient: &Key<key::PublicParts, R>,
+pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
                          S: &Protected,
-                         ciphertext: &Ciphertext)
+                         ciphertext: &mpi::Ciphertext)
     -> Result<SessionKey>
     where R: key::KeyRole
 {
     match (recipient.mpis(), ciphertext) {
-        (PublicKey::ECDH { ref curve, ref hash, ref sym, ..},
-         Ciphertext::ECDH { ref key, .. }) => {
+        (mpi::PublicKey::ECDH { ref curve, ref hash, ref sym, ..},
+         mpi::Ciphertext::ECDH { ref key, .. }) => {
             // Compute KDF input.
             let param = make_param(recipient, curve, hash, sym);
 
@@ -325,51 +112,12 @@ pub fn decrypt_shared<R>(recipient: &Key<key::PublicParts, R>,
     }
 }
 
-fn make_param<P, R>(recipient: &Key<P, R>,
-              curve: &Curve, hash: &HashAlgorithm,
-              sym: &SymmetricAlgorithm)
-    -> Vec<u8>
-    where P: key::KeyParts,
-          R: key::KeyRole
-{
-    // Param = curve_OID_len || curve_OID ||
-    // public_key_alg_ID || 03 || 01 || KDF_hash_ID ||
-    // KEK_alg_ID for AESKeyWrap || "Anonymous Sender    " ||
-    // recipient_fingerprint;
-    let fp = recipient.fingerprint();
-
-    let mut param = Vec::with_capacity(
-        1 + curve.oid().len()        // Length and Curve OID,
-            + 1                      // Public key algorithm ID,
-            + 4                      // KDF parameters,
-            + 20                     // "Anonymous Sender    ",
-            + fp.as_bytes().len());  // Recipients key fingerprint.
-
-    param.push(curve.oid().len() as u8);
-    param.extend_from_slice(curve.oid());
-    param.push(PublicKeyAlgorithm::ECDH.into());
-    param.push(3);
-    param.push(1);
-    param.push((*hash).into());
-    param.push((*sym).into());
-    param.extend_from_slice(b"Anonymous Sender    ");
-    param.extend_from_slice(fp.as_bytes());
-    assert_eq!(param.len(),
-               1 + curve.oid().len()    // Length and Curve OID,
-               + 1                      // Public key algorithm ID,
-               + 4                      // KDF parameters,
-               + 20                     // "Anonymous Sender    ",
-               + fp.as_bytes().len());  // Recipients key fingerprint.
-
-    param
-}
-
 /// Derives a secret key for session key wrapping.
 ///
 /// See [Section 7 of RFC 6637].
 ///
 ///   [Section 7 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-7
-pub fn kdf(x: &Protected, obits: usize, hash: HashAlgorithm, param: &[u8])
+fn kdf(x: &Protected, obits: usize, hash: HashAlgorithm, param: &[u8])
            -> Result<Protected> {
     let mut hash = hash.context()?;
     if obits > hash.digest_size() {
@@ -392,17 +140,21 @@ pub fn kdf(x: &Protected, obits: usize, hash: HashAlgorithm, param: &[u8])
 /// See [Section 8 of RFC 6637].
 ///
 ///   [Section 8 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-8
-pub fn pkcs5_pad(sk: Protected, target_len: usize) -> Protected {
-    let mut buf: Vec<u8> = unsafe {
-        sk.into_vec()
-    };
+fn pkcs5_pad(sk: Protected, target_len: usize) -> Result<Protected> {
+    if sk.len() > target_len {
+        return Err(Error::InvalidArgument(
+            "Plaintext data too large".into()).into());
+    }
+
+    // !!! THIS FUNCTION MUST NOT FAIL FROM THIS POINT ON !!!
+    let mut buf: Vec<u8> = sk.expose_into_unprotected_vec();
     let missing = target_len - buf.len();
     assert!(missing <= 0xff);
     for _ in 0..missing {
         buf.push(missing as u8);
     }
     assert_eq!(buf.len(), target_len);
-    buf.into()
+    Ok(buf.into())
 }
 
 /// Removes PKCS5 padding from a session key.
@@ -410,7 +162,7 @@ pub fn pkcs5_pad(sk: Protected, target_len: usize) -> Protected {
 /// See [Section 8 of RFC 6637].
 ///
 ///   [Section 8 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-8
-pub fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
+fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
     if sk.len() > 0xff {
         return Err(Error::InvalidArgument("message too large".into()).into());
     }
@@ -419,9 +171,7 @@ pub fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
         return Err(Error::InvalidArgument("message too small".into()).into());
     }
 
-    let mut buf: Vec<u8> = unsafe {
-        sk.into_vec()
-    };
+    let mut buf: Vec<u8> = sk.expose_into_unprotected_vec();
     let mut good = true;
     let missing = (buf.len() - target_len) as u8;
     for &b in &buf[target_len..] {
@@ -438,14 +188,15 @@ pub fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
     }
 }
 
+
 /// Wraps a key using the AES Key Wrap Algorithm.
 ///
 /// See [RFC 3394].
 ///
 ///  [RFC 3394]: https://tools.ietf.org/html/rfc3394
-pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
-                    plaintext: &Protected)
-                    -> Result<Vec<u8>> {
+fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
+                plaintext: &Protected)
+                -> Result<Vec<u8>> {
     use crate::SymmetricAlgorithm::*;
 
     if plaintext.len() % 8 != 0 {
@@ -457,17 +208,10 @@ pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    // We need ECB for the algorithm.  However, there is no
-    // nettle::Mode:ECB, and we need nettle::Mode for polymorphism (we
-    // cannot have Box<nettle::Cipher>).  To work around this, we use
-    // CBC, and always use an all-zero IV.
-    let mut cipher: Box<dyn Mode> = match algo {
-        AES128 => Box::new(
-            mode::Cbc::<cipher::Aes128>::with_encrypt_key(key)?),
-        AES192 => Box::new(
-            mode::Cbc::<cipher::Aes192>::with_encrypt_key(key)?),
-        AES256 => Box::new(
-            mode::Cbc::<cipher::Aes256>::with_encrypt_key(key)?),
+    // We need ECB for the algorithm. However, there is no nettle::Mode:ECB,
+    // so to  work around this, we use CBC, and always use an all-zero IV.
+    let mut cipher = match algo {
+        AES128 | AES192 | AES256 => algo.make_encrypt_cbc(key)?,
         _ => return Err(Error::UnsupportedSymmetricAlgorithm(algo).into()),
     };
 
@@ -529,9 +273,9 @@ pub fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
 /// See [RFC 3394].
 ///
 ///  [RFC 3394]: https://tools.ietf.org/html/rfc3394
-pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
-                      ciphertext: &[u8])
-                      -> Result<Protected> {
+fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
+                  ciphertext: &[u8])
+                  -> Result<Protected> {
     use crate::SymmetricAlgorithm::*;
 
     if ciphertext.len() % 8 != 0 {
@@ -543,17 +287,10 @@ pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    // We need ECB for the algorithm.  However, there is no
-    // nettle::Mode:ECB, and we need nettle::Mode for polymorphism (we
-    // cannot have Box<nettle::Cipher>).  To work around this, we use
-    // CBC, and always use an all-zero IV.
-    let mut cipher: Box<dyn Mode> = match algo {
-        AES128 => Box::new(
-            mode::Cbc::<cipher::Aes128>::with_decrypt_key(key)?),
-        AES192 => Box::new(
-            mode::Cbc::<cipher::Aes192>::with_decrypt_key(key)?),
-        AES256 => Box::new(
-            mode::Cbc::<cipher::Aes256>::with_decrypt_key(key)?),
+    // We need ECB for the algorithm. However, there is no nettle::Mode:ECB,
+    // so to  work around this, we use CBC, and always use an all-zero IV.
+    let mut cipher = match algo {
+        AES128 | AES192 | AES256 => algo.make_decrypt_cbc(key)?,
         _ => return Err(Error::UnsupportedSymmetricAlgorithm(algo).into()),
     };
 
@@ -616,6 +353,45 @@ pub fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
     }
 }
 
+fn make_param<P, R>(recipient: &Key<P, R>,
+              curve: &Curve, hash: &HashAlgorithm,
+              sym: &SymmetricAlgorithm)
+    -> Vec<u8>
+    where P: key::KeyParts,
+          R: key::KeyRole
+{
+    // Param = curve_OID_len || curve_OID ||
+    // public_key_alg_ID || 03 || 01 || KDF_hash_ID ||
+    // KEK_alg_ID for AESKeyWrap || "Anonymous Sender    " ||
+    // recipient_fingerprint;
+    let fp = recipient.fingerprint();
+
+    let mut param = Vec::with_capacity(
+        1 + curve.oid().len()        // Length and Curve OID,
+            + 1                      // Public key algorithm ID,
+            + 4                      // KDF parameters,
+            + 20                     // "Anonymous Sender    ",
+            + fp.as_bytes().len());  // Recipients key fingerprint.
+
+    param.push(curve.oid().len() as u8);
+    param.extend_from_slice(curve.oid());
+    param.push(PublicKeyAlgorithm::ECDH.into());
+    param.push(3);
+    param.push(1);
+    param.push((*hash).into());
+    param.push((*sym).into());
+    param.extend_from_slice(b"Anonymous Sender    ");
+    param.extend_from_slice(fp.as_bytes());
+    assert_eq!(param.len(),
+               1 + curve.oid().len()    // Length and Curve OID,
+               + 1                      // Public key algorithm ID,
+               + 4                      // KDF parameters,
+               + 20                     // "Anonymous Sender    ",
+               + fp.as_bytes().len());  // Recipients key fingerprint.
+
+    param
+}
+
 const AES_KEY_WRAP_IV: u64 = 0xa6a6a6a6a6a6a6a6;
 
 #[cfg(test)]
@@ -624,12 +400,12 @@ mod tests {
 
     #[test]
     fn pkcs5_padding() {
-        let v = pkcs5_pad(vec![0, 0, 0].into(), 8);
+        let v = pkcs5_pad(vec![0, 0, 0].into(), 8).unwrap();
         assert_eq!(&v, &Protected::from(&[0, 0, 0, 5, 5, 5, 5, 5][..]));
         let v = pkcs5_unpad(v, 3).unwrap();
         assert_eq!(&v, &Protected::from(&[0, 0, 0][..]));
 
-        let v = pkcs5_pad(vec![].into(), 8);
+        let v = pkcs5_pad(vec![].into(), 8).unwrap();
         assert_eq!(&v, &Protected::from(&[8, 8, 8, 8, 8, 8, 8, 8][..]));
         let v = pkcs5_unpad(v, 0).unwrap();
         assert_eq!(&v, &Protected::from(&[][..]));
