@@ -1,7 +1,17 @@
-//! Functionality to hash packets, and generate hashes.
+//! Cryptographic hash functions and hashing of OpenPGP data
+//! structures.
+//!
+//! This module provides [`Context`] representing a hash function
+//! context independent of the cryptographic backend, as well as trait
+//! [`Hash`] that handles hashing of OpenPGP data structures.
+//!
+//!   [`Context`]: struct.Context.html
+//!   [`Hash`]: trait.Hash.html
 
 use std::convert::TryFrom;
 
+use dyn_clone::DynClone;
+
 use crate::HashAlgorithm;
 use crate::packet::Key;
 use crate::packet::UserID;
@@ -10,7 +20,6 @@ use crate::packet::key;
 use crate::packet::key::Key4;
 use crate::packet::Signature;
 use crate::packet::signature::{self, Signature4};
-use crate::Error;
 use crate::Result;
 use crate::types::Timestamp;
 
@@ -21,13 +30,37 @@ use std::io::{self, Write};
 // hashed to files /tmp/hash-N, where N is a number.
 const DUMP_HASHED_VALUES: Option<&str> = None;
 
+/// Hasher capable of calculating a digest for the input byte stream.
+pub(crate) trait Digest: DynClone {
+    /// Size of the digest in bytes
+    fn digest_size(&self) -> usize;
+
+    /// Writes data into the hash function.
+    fn update(&mut self, data: &[u8]);
+
+    /// Finalizes the hash function and writes the digest into the
+    /// provided slice.
+    ///
+    /// Resets the hash function contexts.
+    ///
+    /// `digest` must be at least `self.digest_size()` bytes large,
+    /// otherwise the digest will be truncated.
+    fn digest(&mut self, digest: &mut [u8]);
+}
+
+dyn_clone::clone_trait_object!(Digest);
+
 /// State of a hash function.
 ///
 /// This provides an abstract interface to the hash functions used in
-/// OpenPGP.
+/// OpenPGP.  `Context`s are created using [`HashAlgorithm::context`].
+///
+///   [`HashAlgorithm::context`]: ../../types/enum.HashAlgorithm.html#method.context
+///
+/// # Examples
 ///
 /// ```rust
-/// # f().unwrap(); fn f() -> sequoia_openpgp::Result<()> {
+/// # fn main() -> sequoia_openpgp::Result<()> {
 /// use sequoia_openpgp::types::HashAlgorithm;
 ///
 /// // Create a context and feed data to it.
@@ -49,7 +82,7 @@ const DUMP_HASHED_VALUES: Option<&str> = None;
 #[derive(Clone)]
 pub struct Context {
     algo: HashAlgorithm,
-    ctx: Box<dyn nettle::hash::Hash>,
+    ctx: Box<dyn Digest>,
 }
 
 impl Context {
@@ -73,8 +106,10 @@ impl Context {
     ///
     /// Resets the hash function contexts.
     ///
-    /// `digest` must be at least `self.digest_size()` bytes large,
+    /// `digest` must be at least [`self.digest_size()`] bytes large,
     /// otherwise the digest will be truncated.
+    ///
+    ///   [`self.digest_size()`]: #method.digest_size
     pub fn digest<D: AsMut<[u8]>>(&mut self, mut digest: D) {
         self.ctx.digest(digest.as_mut());
     }
@@ -92,22 +127,6 @@ impl io::Write for Context {
 }
 
 impl HashAlgorithm {
-    /// Whether Sequoia supports this algorithm.
-    pub fn is_supported(self) -> bool {
-        match self {
-            HashAlgorithm::SHA1 => true,
-            HashAlgorithm::SHA224 => true,
-            HashAlgorithm::SHA256 => true,
-            HashAlgorithm::SHA384 => true,
-            HashAlgorithm::SHA512 => true,
-            HashAlgorithm::RipeMD => true,
-            HashAlgorithm::MD5 => true,
-            HashAlgorithm::Private(_) => false,
-            HashAlgorithm::Unknown(_) => false,
-            HashAlgorithm::__Nonexhaustive => unreachable!(),
-        }
-    }
-
     /// Creates a new hash context for this algorithm.
     ///
     /// # Errors
@@ -118,64 +137,27 @@ impl HashAlgorithm {
     ///
     ///   [`HashAlgorithm::is_supported`]: #method.is_supported
     pub fn context(self) -> Result<Context> {
-        use nettle::hash::{Sha224, Sha256, Sha384, Sha512};
-        use nettle::hash::insecure_do_not_use::{
-            Sha1,
-            Md5,
-            Ripemd160,
-        };
-
-        let c: Result<Box<dyn nettle::hash::Hash>> = match self {
-            HashAlgorithm::SHA1 => Ok(Box::new(Sha1::default())),
-            HashAlgorithm::SHA224 => Ok(Box::new(Sha224::default())),
-            HashAlgorithm::SHA256 => Ok(Box::new(Sha256::default())),
-            HashAlgorithm::SHA384 => Ok(Box::new(Sha384::default())),
-            HashAlgorithm::SHA512 => Ok(Box::new(Sha512::default())),
-            HashAlgorithm::MD5 => Ok(Box::new(Md5::default())),
-            HashAlgorithm::RipeMD => Ok(Box::new(Ripemd160::default())),
-            HashAlgorithm::Private(_) | HashAlgorithm::Unknown(_) =>
-                Err(Error::UnsupportedHashAlgorithm(self).into()),
-            HashAlgorithm::__Nonexhaustive => unreachable!(),
-        };
-
-        c.map(|ctx| Context {
-            algo: self,
-            ctx: if let Some(prefix) = DUMP_HASHED_VALUES {
-                Box::new(HashDumper::new(ctx, prefix))
-            } else {
-                ctx
-            },
-        })
-    }
-
-    /// Returns the ASN.1 OID of this hash algorithm.
-    pub fn oid(self) -> Result<&'static [u8]> {
-        use nettle::rsa;
-
-        match self {
-            HashAlgorithm::SHA1 => Ok(rsa::ASN1_OID_SHA1),
-            HashAlgorithm::SHA224 => Ok(rsa::ASN1_OID_SHA224),
-            HashAlgorithm::SHA256 => Ok(rsa::ASN1_OID_SHA256),
-            HashAlgorithm::SHA384 => Ok(rsa::ASN1_OID_SHA384),
-            HashAlgorithm::SHA512 => Ok(rsa::ASN1_OID_SHA512),
-            HashAlgorithm::MD5 => Ok(rsa::ASN1_OID_MD5),
-            HashAlgorithm::RipeMD => Ok(rsa::ASN1_OID_RIPEMD160),
-            HashAlgorithm::Private(_) | HashAlgorithm::Unknown(_) =>
-                Err(Error::UnsupportedHashAlgorithm(self).into()),
-            HashAlgorithm::__Nonexhaustive => unreachable!(),
-        }
+        self.new_hasher()
+            .map(|hasher| Context {
+                algo: self,
+                ctx: if let Some(prefix) = DUMP_HASHED_VALUES {
+                    Box::new(HashDumper::new(hasher, prefix))
+                } else {
+                    hasher
+                },
+            })
     }
 }
 
 struct HashDumper {
-    h: Box<dyn nettle::hash::Hash>,
+    hasher: Box<dyn Digest>,
     sink: File,
     filename: String,
     written: usize,
 }
 
 impl HashDumper {
-    fn new(h: Box<dyn nettle::hash::Hash>, prefix: &str) -> Self {
+    fn new(hasher: Box<dyn Digest>, prefix: &str) -> Self {
         let mut n = 0;
         let mut filename;
         let sink = loop {
@@ -189,7 +171,7 @@ impl HashDumper {
         };
         eprintln!("HashDumper: Writing to {}...", &filename);
         HashDumper {
-            h,
+            hasher,
             sink,
             filename,
             written: 0,
@@ -197,6 +179,17 @@ impl HashDumper {
     }
 }
 
+impl Clone for HashDumper {
+    fn clone(&self) -> HashDumper {
+        // We only ever create instances of HashDumper when debugging.
+        // Whenever we're cloning an instance, just open another file for
+        // inspection.
+        let prefix = DUMP_HASHED_VALUES
+            .expect("cloning a HashDumper but DUMP_HASHED_VALUES wasn't specified");
+        HashDumper::new(self.hasher.clone(), prefix)
+    }
+}
+
 impl Drop for HashDumper {
     fn drop(&mut self) {
         eprintln!("HashDumper: Wrote {} bytes to {}...", self.written,
@@ -204,31 +197,46 @@ impl Drop for HashDumper {
     }
 }
 
-impl nettle::hash::Hash for HashDumper {
+impl Digest for HashDumper {
     fn digest_size(&self) -> usize {
-        self.h.digest_size()
+        self.hasher.digest_size()
     }
     fn update(&mut self, data: &[u8]) {
-        self.h.update(data);
+        self.hasher.update(data);
         self.sink.write_all(data).unwrap();
         self.written += data.len();
     }
     fn digest(&mut self, digest: &mut [u8]) {
-        self.h.digest(digest);
-    }
-    fn box_clone(&self) -> Box<dyn nettle::hash::Hash> {
-        Box::new(Self::new(self.h.box_clone(), &DUMP_HASHED_VALUES.unwrap()))
+        self.hasher.digest(digest);
     }
 }
 
 /// Hashes OpenPGP packets and related types.
+///
+/// Some OpenPGP data structures need to be hashed to be covered by
+/// OpenPGP signatures.  Hashing is often based on the serialized
+/// form, with some aspects fixed to ensure consistent results.  This
+/// trait implements hashing as specified by OpenPGP.
+///
+/// Most of the time it is not necessary to manually compute hashes.
+/// Instead, higher level functionality, like the streaming
+/// [`Verifier`], [`DetachedVerifier`], or [`Signature`'s verification
+/// functions] should be used, which handle the hashing internally.
+///
+///   [`Verifier`]: ../../parse/stream/struct.Verifier.html
+///   [`DetachedVerifier`]: ../../parse/stream/struct.DetachedVerifier.html
+///   [`Signature`'s verification functions]: ../../packet/enum.Signature.html#verification-functions
+///
+/// This is a low-level mechanism.  See [`Signature`'s hashing
+/// functions] for how to hash compounds like (Key,UserID)-bindings.
+///
+///   [`Signature`'s hashing functions]: ../../packet/enum.Signature.html#hashing-functions
 pub trait Hash {
     /// Updates the given hash with this object.
     fn hash(&self, hash: &mut Context);
 }
 
 impl Hash for UserID {
-    /// Update the Hash with a hash of the user id.
     fn hash(&self, hash: &mut Context) {
         let len = self.value().len() as u32;
 
@@ -242,7 +250,6 @@ impl Hash for UserID {
 }
 
 impl Hash for UserAttribute {
-    /// Update the Hash with a hash of the user attribute.
     fn hash(&self, hash: &mut Context) {
         let len = self.value().len() as u32;
 
@@ -259,7 +266,6 @@ impl<P, R> Hash for Key4<P, R>
     where P: key::KeyParts,
           R: key::KeyRole,
 {
-    /// Update the Hash with a hash of the key.
     fn hash(&self, hash: &mut Context) {
         use crate::serialize::MarshalInto;
 
@@ -296,7 +302,6 @@ impl<P, R> Hash for Key4<P, R>
 }
 
 impl Hash for Signature {
-    /// Adds the `Signature` to the provided hash context.
     fn hash(&self, hash: &mut Context) {
         match self {
             Signature::V4(sig) => sig.hash(hash),
@@ -306,14 +311,12 @@ impl Hash for Signature {
 }
 
 impl Hash for Signature4 {
-    /// Adds the `Signature` to the provided hash context.
     fn hash(&self, hash: &mut Context) {
         self.fields.hash(hash);
     }
 }
 
-impl Hash for signature::SignatureBuilder {
-    /// Adds the `Signature` to the provided hash context.
+impl Hash for signature::SignatureFields {
     fn hash(&self, hash: &mut Context) {
         use crate::serialize::MarshalInto;
 
@@ -372,12 +375,13 @@ impl Hash for signature::SignatureBuilder {
 }
 
 /// Hashing-related functionality.
+///
+/// <a name="hashing-functions"></a>
 impl Signature {
     /// Computes the message digest of standalone signatures.
-    pub fn hash_standalone<'a, S>(sig: S) -> Result<Vec<u8>>
-        where S: Into<&'a signature::SignatureBuilder>
+    pub fn hash_standalone(sig: &signature::SignatureFields)
+        -> Result<Vec<u8>>
     {
-        let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
 
         sig.hash(&mut h);
@@ -388,21 +392,20 @@ impl Signature {
     }
 
     /// Computes the message digest of timestamp signatures.
-    pub fn hash_timestamp<'a, S>(sig: S) -> Result<Vec<u8>>
-        where S: Into<&'a signature::SignatureBuilder>
+    pub fn hash_timestamp(sig: &signature::SignatureFields)
+        -> Result<Vec<u8>>
     {
         Self::hash_standalone(sig)
     }
 
     /// Returns the message digest of the direct key signature over
     /// the specified primary key.
-    pub fn hash_direct_key<'a, P, S>(sig: S, key: &Key<P, key::PrimaryRole>)
+    pub fn hash_direct_key<P>(sig: &signature::SignatureFields,
+                              key: &Key<P, key::PrimaryRole>)
         -> Result<Vec<u8>>
         where P: key::KeyParts,
-              S: Into<&'a signature::SignatureBuilder>,
     {
 
-        let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
 
         key.hash(&mut h);
@@ -415,17 +418,13 @@ impl Signature {
 
     /// Returns the message digest of the subkey binding over the
     /// specified primary key and subkey.
-    pub fn hash_subkey_binding<'a, P, Q, S>(
-        sig: S,
-        key: &Key<P, key::PrimaryRole>,
-        subkey: &Key<Q, key::SubordinateRole>)
+    pub fn hash_subkey_binding<P, Q>(sig: &signature::SignatureFields,
+                                     key: &Key<P, key::PrimaryRole>,
+                                     subkey: &Key<Q, key::SubordinateRole>)
         -> Result<Vec<u8>>
         where P: key::KeyParts,
               Q: key::KeyParts,
-              S: Into<&'a signature::SignatureBuilder>
     {
-
-        let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
 
         key.hash(&mut h);
@@ -439,28 +438,24 @@ impl Signature {
 
     /// Returns the message digest of the primary key binding over the
     /// specified primary key and subkey.
-    pub fn hash_primary_key_binding<'a, P, Q, S>(
-        sig: S,
-        key: &Key<P, key::PrimaryRole>,
-        subkey: &Key<Q, key::SubordinateRole>)
+    pub fn hash_primary_key_binding<P, Q>(sig: &signature::SignatureFields,
+                                          key: &Key<P, key::PrimaryRole>,
+                                          subkey: &Key<Q, key::SubordinateRole>)
         -> Result<Vec<u8>>
         where P: key::KeyParts,
               Q: key::KeyParts,
-              S: Into<&'a signature::SignatureBuilder>
     {
-        Self::hash_subkey_binding(sig.into(), key, subkey)
+        Self::hash_subkey_binding(sig, key, subkey)
     }
 
     /// Returns the message digest of the user ID binding over the
     /// specified primary key, user ID, and signature.
-    pub fn hash_userid_binding<'a, P, S>(sig: S,
-                                         key: &Key<P, key::PrimaryRole>,
-                                         userid: &UserID)
+    pub fn hash_userid_binding<P>(sig: &signature::SignatureFields,
+                                  key: &Key<P, key::PrimaryRole>,
+                                  userid: &UserID)
         -> Result<Vec<u8>>
         where P: key::KeyParts,
-              S: Into<&'a signature::SignatureBuilder>
     {
-        let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
 
         key.hash(&mut h);
@@ -474,16 +469,13 @@ impl Signature {
 
     /// Returns the message digest of the user attribute binding over
     /// the specified primary key, user attribute, and signature.
-    pub fn hash_user_attribute_binding<'a, P, S>(
-        sig: S,
+    pub fn hash_user_attribute_binding<P>(
+        sig: &signature::SignatureFields,
         key: &Key<P, key::PrimaryRole>,
         ua: &UserAttribute)
         -> Result<Vec<u8>>
         where P: key::KeyParts,
-              S: Into<&'a signature::SignatureBuilder>,
     {
-
-        let sig = sig.into();
         let mut h = sig.hash_algo().context()?;
 
         key.hash(&mut h);
