@@ -1,10 +1,30 @@
 //! Memory protection and encryption.
+//!
+//! Sequoia makes an effort to protect secrets stored in memory.  Even
+//! though a process's memory should be protected from being read by an
+//! adversary, there may be bugs in the program or the architecture
+//! the program is running on that allow (partial) recovery of data.
+//! Or, the process may be serialized to persistent storage, and its
+//! memory may be inspected while it is not running.
+//!
+//! To reduce the window for these kind of exfiltrations, we use
+//! [`Protected`] to clear the memory once it is no longer in use, and
+//! [`Encrypted`] to protect long-term secrets like passwords and
+//! secret keys.
+//!
+//!   [`Protected`]: struct.Protected.html
+//!   [`Encrypted`]: struct.Encrypted.html
+//!
+//! Furthermore, operations involving secrets must be carried out in a
+//! way that avoids leaking information.  For example, comparison
+//! must be done in constant time with [`secure_cmp`].
+//!
+//!   [`secure_cmp`]: fn.secure_cmp.html
 
 use std::cmp::{min, Ordering};
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::ops::{Deref, DerefMut};
-use std::pin::Pin;
 
 use memsec;
 
@@ -13,6 +33,8 @@ use memsec;
 /// The memory is guaranteed not to be copied around, and is cleared
 /// when the object is dropped.
 ///
+/// # Examples
+///
 /// ```rust
 /// use sequoia_openpgp::crypto::mem::Protected;
 ///
@@ -23,12 +45,37 @@ use memsec;
 ///
 /// // p is cleared once it goes out of scope.
 /// ```
-#[derive(Clone)]
-pub struct Protected(Pin<Box<[u8]>>);
+// # Note on the implementation
+//
+// We use a boxed slice, then Box::leak the Box.  This takes the
+// knowledge about the shape of the heap allocation away from Rust,
+// preventing any optimization based on that.
+//
+// For example, Rust could conceivably compact the heap: The borrow
+// checker knows when no references exist, and this is an excellent
+// opportunity to move the object on the heap because only one pointer
+// needs to be updated.
+pub struct Protected(*mut [u8]);
+
+// Safety: Box<[u8]> is Send and Sync, we do not expose any
+// functionality that was not possible before, hence Protected may
+// still be Send and Sync.
+unsafe impl Send for Protected {}
+unsafe impl Sync for Protected {}
+
+impl Clone for Protected {
+    fn clone(&self) -> Self {
+        // Make a vector with the correct size to avoid potential
+        // reallocations when turning it into a `Protected`.
+        let mut p = Vec::with_capacity(self.len());
+        p.extend_from_slice(&self);
+        p.into_boxed_slice().into()
+    }
+}
 
 impl PartialEq for Protected {
     fn eq(&self, other: &Self) -> bool {
-        secure_cmp(&self.0, &other.0) == Ordering::Equal
+        secure_cmp(&self, &other) == Ordering::Equal
     }
 }
 
@@ -36,14 +83,18 @@ impl Eq for Protected {}
 
 impl Hash for Protected {
     fn hash<H: Hasher>(&self, state: &mut H) {
-        self.0.hash(state);
+        self.as_ref().hash(state);
     }
 }
 
 impl Protected {
     /// Converts to a buffer for modification.
-    pub(crate) unsafe fn into_vec(self) -> Vec<u8> {
-        self.iter().cloned().collect()
+    ///
+    /// Don't expose `Protected` values unless you know what you're doing.
+    pub(crate) fn expose_into_unprotected_vec(self) -> Vec<u8> {
+        let mut p = Vec::with_capacity(self.len());
+        p.extend_from_slice(&self);
+        p
     }
 }
 
@@ -51,37 +102,52 @@ impl Deref for Protected {
     type Target = [u8];
 
     fn deref(&self) -> &Self::Target {
-        &self.0
+        self.as_ref()
     }
 }
 
 impl AsRef<[u8]> for Protected {
     fn as_ref(&self) -> &[u8] {
-        &self.0
+        unsafe { &*self.0 }
     }
 }
 
 impl AsMut<[u8]> for Protected {
     fn as_mut(&mut self) -> &mut [u8] {
-        &mut self.0
+        unsafe { &mut *self.0 }
     }
 }
 
 impl DerefMut for Protected {
     fn deref_mut(&mut self) -> &mut [u8] {
-        &mut self.0
+        self.as_mut()
     }
 }
 
 impl From<Vec<u8>> for Protected {
-    fn from(v: Vec<u8>) -> Self {
-        Protected(Pin::new(v.into_boxed_slice()))
+    fn from(mut v: Vec<u8>) -> Self {
+        // Make a vector with the correct size to avoid potential
+        // reallocations when turning it into a `Protected`.
+        let mut p = Vec::with_capacity(v.len());
+        p.extend_from_slice(&v);
+
+        // Now clear the previous allocation.  Just to be safe, we
+        // clear the whole allocation.
+        let capacity = v.capacity();
+        unsafe {
+            // Safety: New size is equal to the capacity, and we
+            // initialize all elements.
+            v.set_len(capacity);
+            memsec::memzero(v.as_mut_ptr(), capacity);
+        }
+
+        p.into_boxed_slice().into()
     }
 }
 
 impl From<Box<[u8]>> for Protected {
     fn from(v: Box<[u8]>) -> Self {
-        Protected(Pin::new(v))
+        Protected(Box::leak(v))
     }
 }
 
@@ -94,7 +160,9 @@ impl From<&[u8]> for Protected {
 impl Drop for Protected {
     fn drop(&mut self) {
         unsafe {
-            memsec::memzero(self.0.as_mut_ptr(), self.0.len());
+            let len = self.len();
+            memsec::memzero(self.as_mut().as_mut_ptr(), len);
+            Box::from_raw(self.0);
         }
     }
 }
@@ -133,7 +201,7 @@ impl fmt::Debug for Protected {
 /// adding it can be found
 /// [here](https://marc.info/?l=openbsd-cvs&m=156109087822676).
 ///
-/// # Example
+/// # Examples
 ///
 /// ```rust
 /// use sequoia_openpgp::crypto::mem::Encrypted;
@@ -273,10 +341,9 @@ pub fn secure_cmp(a: &[u8], b: &[u8]) -> Ordering {
         memsec::memcmp(a.as_ptr(), b.as_ptr(), min(a.len(), b.len()))
     };
     let ord2 = match ord2 {
+        1..=std::i32::MAX => Ordering::Greater,
         0 => Ordering::Equal,
-        a if a < 0 => Ordering::Less,
-        a if a > 0 => Ordering::Greater,
-        _ => unreachable!(),
+        std::i32::MIN..=-1 => Ordering::Less,
     };
 
     if ord1 == Ordering::Equal { ord2 } else { ord1 }
