@@ -1,6 +1,6 @@
 use std::fmt;
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
 /// A long identifier for certificates and keys.
@@ -92,7 +92,7 @@ impl Fingerprint {
     /// object with the `:X` format specifier.
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// use openpgp::Fingerprint;
     ///
     /// let fpr = "0123 4567 89AB CDEF 0123 4567 89AB CDEF 0123 4567".parse::<Fingerprint>().unwrap();
@@ -111,7 +111,7 @@ impl Fingerprint {
     /// optional leading `0x`.
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// use openpgp::Fingerprint;
     ///
     /// let fpr = Fingerprint::from_hex("0123456789ABCDEF0123456789ABCDEF01234567").unwrap();
@@ -216,7 +216,7 @@ impl Fingerprint {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for Fingerprint {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         use rand::Rng;
