@@ -13,7 +13,7 @@ use crate::{
 /// Identifies certificates and keys.
 ///
 /// A `KeyHandle` is either a `Fingerprint` or a `KeyID`.
-#[derive(Debug, Clone, Hash)]
+#[derive(Debug, Clone)]
 pub enum KeyHandle {
     /// A Fingerprint.
     Fingerprint(Fingerprint),
@@ -185,7 +185,7 @@ impl KeyHandle {
     /// non-transitive equality relation:
     ///
     /// ```
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// # use openpgp::Fingerprint;
     /// # use openpgp::KeyID;
     /// # use openpgp::KeyHandle;
