@@ -1,6 +1,6 @@
 use std::fmt;
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
 use crate::Error;
@@ -183,7 +183,7 @@ impl KeyID {
     /// object with the `:X` format specifier.
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// use openpgp::KeyID;
     ///
     /// let keyid = "fb3751f1587daef1".parse::<KeyID>().unwrap();
@@ -202,7 +202,7 @@ impl KeyID {
     /// leading `0x`.
     ///
     /// ```rust
-    /// # extern crate sequoia_openpgp as openpgp;
+    /// # use sequoia_openpgp as openpgp;
     /// use openpgp::KeyID;
     ///
     /// let keyid = KeyID::from_hex("0xfb3751f1587daef1").unwrap();
@@ -262,7 +262,7 @@ impl KeyID {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for KeyID {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         KeyID::new(u64::arbitrary(g))
