@@ -48,7 +48,7 @@ pub enum Token {
 
 impl fmt::Display for Token {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.write_str(&format!("{:?}", self)[..])
+        write!(f, "{:?}", self)
     }
 }
 
@@ -59,7 +59,7 @@ pub enum LexicalError {
 
 impl fmt::Display for LexicalError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.write_str(&format!("{:?}", self)[..])
+        write!(f, "{:?}", self)
     }
 }
 
