@@ -1,6 +1,6 @@
 use std::fmt;
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
 use crate::packet;
@@ -96,7 +96,7 @@ impl From<CompressedData> for Packet {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for CompressedData {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         use rand::Rng;
