@@ -161,7 +161,7 @@ use std::ops::{Deref, DerefMut};
 use std::slice;
 use std::iter::IntoIterator;
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
 use crate::Error;
@@ -295,7 +295,7 @@ macro_rules! impl_into_iterator {
     };
     ($t:ty where $( $w:ident: $c:path ),*) => {
         /// Implement `IntoIterator` so that
-        /// `cert::merge_packets(sig)` just works.
+        /// `cert::insert_packets(sig)` just works.
         impl<$($w),*> IntoIterator for $t
             where $($w: $c ),*
         {
@@ -439,7 +439,7 @@ impl<'a> DerefMut for Packet {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for Packet {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         use rand::Rng;
@@ -485,7 +485,7 @@ pub struct Common {
     dummy: std::marker::PhantomData<()>,
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for Common {
     fn arbitrary<G: Gen>(_: &mut G) -> Self {
         // XXX: Change if this gets interesting fields.
@@ -639,7 +639,7 @@ impl<'a> Iter<'a> {
     /// #
     /// #     let mut cd = CompressedData::new(
     /// #         CompressionAlgorithm::Uncompressed);
-    /// #     cd.set_body(packet::Body::Structured(vec![ lit.clone() ]));
+    /// #     cd.set_body(packet::Body::Structured(vec![lit.clone()]));
     /// #     let cd = Packet::from(cd);
     /// #
     /// #     // Make sure we created the message correctly: serialize,
@@ -650,9 +650,9 @@ impl<'a> Iter<'a> {
     /// #     let pp = PacketPile::from_bytes(&bytes[..])?;
     /// #
     /// #     assert_eq!(pp.descendants().count(), 2);
-    /// #     assert_eq!(pp.path_ref(&[ 0 ]).unwrap().tag(),
+    /// #     assert_eq!(pp.path_ref(&[0]).unwrap().tag(),
     /// #                packet::Tag::CompressedData);
-    /// #     assert_eq!(pp.path_ref(&[ 0, 0 ]), Some(&lit));
+    /// #     assert_eq!(pp.path_ref(&[0, 0]), Some(&lit));
     /// #
     /// #     cd
     /// # };
@@ -872,11 +872,11 @@ fn packet_path_iter() {
 ///     .set_signature_creation_time(t2)?;
 /// let sig = userid.bind(&mut signer, &cert, sig)?;
 ///
-/// let cert = cert.merge_packets(vec![ Packet::from(userid), sig.into() ])?;
+/// let cert = cert.insert_packets(vec![Packet::from(userid), sig.into()])?;
 /// # assert_eq!(cert.with_policy(p, t2)?.userids().count(), 2);
 /// # Ok(()) }
 /// ```
-#[derive(PartialEq, Eq, Hash, Clone, Debug)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Debug)]
 pub enum Signature {
     /// Signature packet version 4.
     V4(self::signature::Signature4),
@@ -1312,7 +1312,7 @@ impl From<SKESK> for Packet {
 /// let sigs = cert.set_expiration_time(p, None, &mut keypair,
 ///                                     Some(time::SystemTime::now()))?;
 ///
-/// let cert = cert.merge_packets(sigs)?;
+/// let cert = cert.insert_packets(sigs)?;
 /// // It's expired now.
 /// assert!(cert.with_policy(p, None)?.alive().is_err());
 /// # Ok(())
@@ -1656,10 +1656,10 @@ impl<R: key::KeyRole> Key<key::SecretParts, R> {
     /// let key = key.encrypt_secret(&"1234".into())?;
     /// assert!(! key.has_unencrypted_secret());
     ///
-    /// // Merge it into the certificate.  Note: `Cert::merge_packets`
+    /// // Merge it into the certificate.  Note: `Cert::insert_packets`
     /// // prefers added versions of keys.  So, the encrypted version
     /// // will override the decrypted version.
-    /// let cert = cert.merge_packets(Packet::from(key))?;
+    /// let cert = cert.insert_packets(Packet::from(key))?;
     ///
     /// // Now the primary key's secret key material is encrypted.
     /// let key = cert.primary_key().key().parts_as_secret()?;
