@@ -5,7 +5,7 @@
 //!
 //!   [Section 5.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.1
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
 use crate::Error;
@@ -190,14 +190,14 @@ impl From<PKESK3> for Packet {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for super::PKESK {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         PKESK3::arbitrary(g).into()
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for PKESK3 {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         let (ciphertext, pk_algo) = loop {
@@ -358,46 +358,37 @@ mod tests {
     fn decrypt_with_short_cv25519_secret_key() {
         use super::PKESK3;
         use crate::crypto::SessionKey;
-        use crate::crypto::mpi::{self, MPI};
-        use crate::PublicKeyAlgorithm;
-        use crate::SymmetricAlgorithm;
-        use crate::HashAlgorithm;
-        use crate::types::Curve;
-        use crate::packet::key;
-        use crate::packet::key::Key4;
-        use nettle::curve25519;
+        use crate::{HashAlgorithm, SymmetricAlgorithm};
+        use crate::packet::key::{Key4, UnspecifiedRole};
 
         // 20 byte sec key
-        let mut sec = [
+        let mut secret_key = [
             0x0,0x0,
             0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
             0x1,0x2,0x2,0x2,0x2,0x2,0x2,0x2,0x2,0x2,
             0x1,0x2,0x2,0x2,0x2,0x2,0x2,0x2,0x0,0x0
         ];
-        let mut pnt = [0x40u8; curve25519::CURVE25519_SIZE + 1];
-        curve25519::mul_g(&mut pnt[1..], &sec[..]).unwrap();
-        sec.reverse();
+        // Ensure that the key is at least somewhat valid, according to the
+        // generation procedure specified in "Responsibilities of the user":
+        // https://cr.yp.to/ecdh/curve25519-20060209.pdf#page=5
+        // Only perform the bit-twiddling on the last byte. This is done so that
+        // we can still have somewhat defined multiplication while still testing
+        // the "short" key logic.
+        // secret_key[0] &= 0xf8;
+        secret_key[31] &= 0x7f;
+        secret_key[31] |= 0x40;
+
+        let key: Key<_, UnspecifiedRole> = Key4::import_secret_cv25519(
+            &secret_key,
+            HashAlgorithm::SHA256,
+            SymmetricAlgorithm::AES256,
+            None,
+        ).unwrap().into();
 
-        let public_mpis = mpi::PublicKey::ECDH {
-            curve: Curve::Cv25519,
-            q: MPI::new(&pnt[..]),
-            hash: HashAlgorithm::SHA256,
-            sym: SymmetricAlgorithm::AES256,
-        };
-        let private_mpis = mpi::SecretKeyMaterial::ECDH {
-            scalar: MPI::new(&sec[..]).into(),
-        };
-        let key: key::UnspecifiedPublic
-            = Key4::new(std::time::SystemTime::now(),
-                        PublicKeyAlgorithm::ECDH,
-                        public_mpis)
-                .unwrap().into();
-        let key = key.add_secret(private_mpis.into()).0;
         let sess_key = SessionKey::new(32);
         let pkesk = PKESK3::for_recipient(SymmetricAlgorithm::AES256, &sess_key,
                                           &key).unwrap();
-        let mut keypair =
-            key.parts_into_secret().unwrap().into_keypair().unwrap();
+        let mut keypair = key.into_keypair().unwrap();
         pkesk.decrypt(&mut keypair, None).unwrap();
     }
 }
