@@ -6,9 +6,9 @@
 
 use std::fmt;
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 use rand::Rng;
 
 use buffered_reader::BufferedReader;
@@ -103,7 +103,7 @@ impl From<UserAttribute> for Packet {
     }
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for UserAttribute {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
         UserAttribute::new(
@@ -192,13 +192,17 @@ pub enum Subpacket {
     Unknown(u8, Box<[u8]>),
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for Subpacket {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 2) {
+        match g.gen_range(0, 3) {
             0 => Subpacket::Image(Image::arbitrary(g)),
             1 => Subpacket::Unknown(
-                g.gen_range(1, 256) as u8,
+                0,
+                Vec::<u8>::arbitrary(g).into_boxed_slice()
+            ),
+            2 => Subpacket::Unknown(
+                g.gen_range(2, 256) as u8,
                 Vec::<u8>::arbitrary(g).into_boxed_slice()
             ),
             _ => unreachable!(),
@@ -221,17 +225,17 @@ pub enum Image {
     Unknown(u8, Box<[u8]>),
 }
 
-#[cfg(any(test, feature = "quickcheck"))]
+#[cfg(test)]
 impl Arbitrary for Image {
     fn arbitrary<G: Gen>(g: &mut G) -> Self {
-        match g.gen_range(0, 3) {
+        match g.gen_range(0, 5) {
             0 =>
                 Image::JPEG(
                     Vec::<u8>::arbitrary(g).into_boxed_slice()
                 ),
             1 =>
                 Image::Unknown(
-                    g.gen_range(1, 100),
+                    g.gen_range(2, 100),
                     Vec::<u8>::arbitrary(g).into_boxed_slice()
                 ),
             2 =>
@@ -239,6 +243,16 @@ impl Arbitrary for Image {
                     g.gen_range(100, 111),
                     Vec::<u8>::arbitrary(g).into_boxed_slice()
                 ),
+            3 =>
+                Image::Unknown(
+                    0,
+                    Vec::<u8>::arbitrary(g).into_boxed_slice()
+                ),
+            4 =>
+                Image::Unknown(
+                    g.gen_range(111, 256) as u8,
+                    Vec::<u8>::arbitrary(g).into_boxed_slice()
+                ),
             _ => unreachable!(),
         }
     }
@@ -252,12 +266,56 @@ mod tests {
 
     quickcheck! {
         fn roundtrip(p: UserAttribute) -> bool {
-            let q = UserAttribute::from_bytes(&p.to_vec().unwrap()).unwrap();
+            let buf = p.to_vec().unwrap();
+            assert_eq!(p.serialized_len(), buf.len());
+            let q = UserAttribute::from_bytes(&buf).unwrap();
             assert_eq!(p, q);
             true
         }
     }
 
+    quickcheck! {
+        fn roundtrip_subpacket(sp: Subpacket) -> bool {
+            let value = sp.to_vec().unwrap();
+            assert_eq!(sp.serialized_len(), value.len());
+            let ua = UserAttribute {
+                common: Default::default(),
+                value,
+            };
+            let buf = ua.to_vec().unwrap();
+            let q = UserAttribute::from_bytes(&buf).unwrap();
+            let subpackets = q.subpackets().collect::<Vec<_>>();
+            assert_eq!(subpackets.len(), 1);
+            assert_eq!(&sp, subpackets[0].as_ref().unwrap());
+            true
+        }
+    }
+
+    quickcheck! {
+        fn roundtrip_image(img: Image) -> bool {
+            let mut body = img.to_vec().unwrap();
+            assert_eq!(img.serialized_len(), body.len());
+            let mut value =
+                BodyLength::Full(1 + body.len() as u32).to_vec().unwrap();
+            value.push(1); // Image subpacket tag.
+            value.append(&mut body);
+            let ua = UserAttribute {
+                common: Default::default(),
+                value,
+            };
+            let buf = ua.to_vec().unwrap();
+            let q = UserAttribute::from_bytes(&buf).unwrap();
+            let subpackets = q.subpackets().collect::<Vec<_>>();
+            assert_eq!(subpackets.len(), 1);
+            if let Ok(Subpacket::Image(i)) = &subpackets[0] {
+                assert_eq!(&img, i);
+            } else {
+                panic!("expected image subpacket, got {:?}", subpackets[0]);
+            }
+            true
+        }
+    }
+
     #[test]
     fn image() {
         use crate::Packet;
@@ -286,11 +344,27 @@ ABQQAQAAAAAAAAAAAAAAAAAAAAD/2gAIAQEAAT8Qf//Z
         };
         assert_eq!(subpackets.len(), 1);
         if let Ok(Subpacket::Image(Image::JPEG(img))) = &subpackets[0] {
-            assert_eq!(img.len(), 539);
+            assert_eq!(img.len(), 539 /* Image data */);
             assert_eq!(&img[6..10], b"JFIF");
             assert_eq!(&img[24..41], b"Created with GIMP");
         } else {
             panic!("Expected JPEG, got {:?}", &subpackets[0]);
         }
+
+        if let Ok(Subpacket::Image(img)) = &subpackets[0] {
+            let buf = img.to_vec().unwrap();
+            assert_eq!(buf.len(), 539 + 16 /* Image header */);
+            assert_eq!(img.serialized_len(), 539 + 16 /* Image header */);
+        } else {
+            unreachable!("decomposed fine before");
+        }
+
+        if let Ok(img) = &subpackets[0] {
+            let buf = img.to_vec().unwrap();
+            assert_eq!(buf.len(), 539 + 16 + 3 /* Subpacket header */);
+            assert_eq!(img.serialized_len(), 539 + 16 + 3 /* Subpacket header */);
+        } else {
+            unreachable!("decomposed fine before");
+        }
     }
 }
