@@ -1,6 +1,6 @@
 use crate::Result;
 use crate::cert::prelude::*;
-use crate::packet::{key, Signature, Tag};
+use crate::packet::{header::BodyLength, key, Signature, Tag};
 use crate::serialize::{
     PacketRef,
     Marshal, MarshalInto,
@@ -271,11 +271,11 @@ impl MarshalInto for Cert {
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
-        generic_serialize_into(self, buf)
+        generic_serialize_into(self, self.serialized_len(), buf)
     }
 
     fn export_into(&self, buf: &mut [u8]) -> Result<usize> {
-        generic_export_into(self, buf)
+        generic_export_into(self, self.serialized_len(), buf)
     }
 }
 
@@ -290,7 +290,7 @@ impl Cert {
     }
 }
 
-/// A reference to a Cert that allows serialization of secret keys.
+/// A reference to a `Cert` that allows serialization of secret keys.
 ///
 /// To avoid accidental leakage, secret keys are not serialized when a
 /// serializing a [`Cert`].  To serialize [`Cert`]s with secret keys,
@@ -300,11 +300,11 @@ impl Cert {
 /// [`Cert`]: ../cert/struct.Cert.html
 /// [`Cert::as_tsk()`]: ../cert/struct.Cert.html#method.as_tsk
 ///
-/// # Example
+/// # Examples
+///
 /// ```
 /// # use sequoia_openpgp::{*, cert::*, parse::Parse, serialize::Serialize};
-/// # f().unwrap();
-/// # fn f() -> Result<()> {
+/// # fn main() -> Result<()> {
 /// let (cert, _) = CertBuilder::new().generate()?;
 /// assert!(cert.is_tsk());
 ///
@@ -315,9 +315,11 @@ impl Cert {
 /// assert!(cert_.is_tsk());
 /// assert_eq!(cert, cert_);
 /// # Ok(()) }
+/// ```
 pub struct TSK<'a> {
     cert: &'a Cert,
     filter: Option<Box<dyn Fn(&'a key::UnspecifiedSecret) -> bool + 'a>>,
+    emit_stubs: bool,
 }
 
 impl<'a> TSK<'a> {
@@ -326,6 +328,7 @@ impl<'a> TSK<'a> {
         Self {
             cert,
             filter: None,
+            emit_stubs: false,
         }
     }
 
@@ -333,13 +336,16 @@ impl<'a> TSK<'a> {
     ///
     /// Note that the given filter replaces any existing filter.
     ///
-    /// # Example
+    /// # Examples
+    ///
+    /// This example demonstrates how to create a TSK with a detached
+    /// primary secret key.
+    ///
     /// ```
     /// # use sequoia_openpgp::{*, cert::*, parse::Parse, serialize::Serialize};
     /// use sequoia_openpgp::policy::StandardPolicy;
     ///
-    /// # f().unwrap();
-    /// # fn f() -> Result<()> {
+    /// # fn main() -> Result<()> {
     /// let p = &StandardPolicy::new();
     ///
     /// let (cert, _) = CertBuilder::new().add_signing_subkey().generate()?;
@@ -348,13 +354,14 @@ impl<'a> TSK<'a> {
     /// // Only write out the primary key's secret.
     /// let mut buf = Vec::new();
     /// cert.as_tsk()
-    ///     .set_filter(|k| k.fingerprint() == cert.fingerprint())
+    ///     .set_filter(|k| k.fingerprint() != cert.fingerprint())
     ///     .serialize(&mut buf)?;
     ///
     /// let cert_ = Cert::from_bytes(&buf)?;
+    /// assert!(! cert_.primary_key().has_secret());
     /// assert_eq!(cert_.keys().with_policy(p, None).alive().revoked(false).secret().count(), 1);
-    /// assert!(cert_.primary_key().has_secret());
     /// # Ok(()) }
+    /// ```
     pub fn set_filter<P>(mut self, predicate: P) -> Self
         where P: 'a + Fn(&'a key::UnspecifiedSecret) -> bool
     {
@@ -362,6 +369,83 @@ impl<'a> TSK<'a> {
         self
     }
 
+    /// Changes `TSK` to emit secret key stubs.
+    ///
+    /// If [`TSK::set_filter`] is used to selectively export secret
+    /// keys, or if the cert contains both keys without secret key
+    /// material and with secret key material, then are two ways to
+    /// serialize this cert.  Neither is sanctioned by the OpenPGP
+    /// standard.
+    ///
+    /// The default way is to simply emit public key packets when no
+    /// secret key material is available.  While straight forward,
+    /// this may be in violation of [Section 11.2 of RFC 4880].
+    ///
+    /// The alternative is to emit a secret key packet with a
+    /// placeholder secret key value.  GnuPG uses this variant with a
+    /// private [`S2K`] format.  If interoperability with GnuPG is a
+    /// concern, use this variant.
+    ///
+    /// See [this test] for support in other implementations.
+    ///
+    ///   [`TSK::set_filter`]: #method.set_filter
+    ///   [Section 11.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.2
+    ///   [`S2K`]: ../crypto/enum.S2K.html
+    ///   [this test]: https://tests.sequoia-pgp.org/#Detached_primary_key
+    ///
+    /// # Examples
+    ///
+    /// This example demonstrates how to create a TSK with a detached
+    /// primary secret key, serializing it using secret key stubs.
+    ///
+    /// ```
+    /// # fn main() -> sequoia_openpgp::Result<()> {
+    /// # use std::convert::TryFrom;
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::packet::key::*;
+    /// # use openpgp::{types::*, crypto::S2K};
+    /// # use openpgp::{*, cert::*, parse::Parse, serialize::Serialize};
+    ///
+    /// let p = &openpgp::policy::StandardPolicy::new();
+    ///
+    /// let (cert, _) = CertBuilder::new().add_signing_subkey().generate()?;
+    /// assert_eq!(cert.keys().with_policy(p, None)
+    ///            .alive().revoked(false).unencrypted_secret().count(), 2);
+    ///
+    /// // Only write out the primary key's secret.
+    /// let mut buf = Vec::new();
+    /// cert.as_tsk()
+    ///     .set_filter(|k| k.fingerprint() != cert.fingerprint())
+    ///     .emit_secret_key_stubs(true)
+    ///     .serialize(&mut buf)?;
+    ///
+    /// # let pp = PacketPile::from_bytes(&buf)?;
+    /// # assert_eq!(pp.path_ref(&[0]).unwrap().kind(),
+    /// #            Some(packet::Tag::SecretKey));
+    /// let cert_ = Cert::from_bytes(&buf)?;
+    /// // The primary key has an "encrypted" stub.
+    /// assert!(cert_.primary_key().has_secret());
+    /// assert_eq!(cert_.keys().with_policy(p, None)
+    ///            .alive().revoked(false).unencrypted_secret().count(), 1);
+    /// # if let Some(SecretKeyMaterial::Encrypted(sec)) =
+    /// #     cert_.primary_key().optional_secret()
+    /// # {
+    /// #     assert_eq!(sec.algo(), SymmetricAlgorithm::Unencrypted);
+    /// #     if let S2K::Private { tag, .. } = sec.s2k() {
+    /// #         assert_eq!(*tag, 101);
+    /// #     } else {
+    /// #         panic!("expected proprietary S2K type");
+    /// #     }
+    /// # } else {
+    /// #     panic!("expected ''encrypted'' secret key stub");
+    /// # }
+    /// # Ok(()) }
+    /// ```
+    pub fn emit_secret_key_stubs(mut self, emit_stubs: bool) -> Self {
+        self.emit_stubs = emit_stubs;
+        self
+    }
+
     /// Serializes or exports the Cert.
     ///
     /// If `export` is true, then non-exportable signatures are not
@@ -396,6 +480,39 @@ impl<'a> TSK<'a> {
                 tag_public
             };
 
+            if self.emit_stubs && (tag == Tag::PublicKey
+                                   || tag == Tag::PublicSubkey) {
+                // Emit a GnuPG-style secret key stub.
+                let stub = crate::crypto::S2K::Private {
+                    tag: 101,
+                    parameters: Some(vec![
+                        0,    // "hash algo"
+                        0x47, // 'G'
+                        0x4e, // 'N'
+                        0x55, // 'U'
+                        1     // "mode"
+                    ].into()),
+                };
+                let key_with_stub = key.clone()
+                    .add_secret(key::SecretKeyMaterial::Encrypted(
+                        key::Encrypted::new(
+                            stub, 0.into(),
+                            // Mirrors more closely what GnuPG 2.1
+                            // does (oddly, GnuPG 1.4 emits 0xfe
+                            // here).
+                            Some(crate::crypto::mpi::SecretKeyChecksum::Sum16),
+                            vec![].into()))).0;
+                return match tag {
+                    Tag::PublicKey =>
+                        crate::Packet::SecretKey(key_with_stub.into())
+                            .serialize(o),
+                    Tag::PublicSubkey =>
+                        crate::Packet::SecretSubkey(key_with_stub.into())
+                            .serialize(o),
+                    _ => unreachable!(),
+                };
+            }
+
             match tag {
                 Tag::PublicKey =>
                     PacketRef::PublicKey(key.into()).serialize(o),
@@ -557,6 +674,16 @@ impl<'a> MarshalInto for TSK<'a> {
                 tag_public
             };
 
+            if self.emit_stubs && (tag == Tag::PublicKey
+                                   || tag == Tag::PublicSubkey) {
+                // Emit a GnuPG-style secret key stub.  The stub
+                // extends the public key by 8 bytes.
+                let l = key.net_len_key(false) + 8;
+                return 1 // CTB
+                    + BodyLength::Full(l as u32).serialized_len()
+                    + l;
+            }
+
             let packet = match tag {
                 Tag::PublicKey => PacketRef::PublicKey(key.into()),
                 Tag::PublicSubkey => PacketRef::PublicSubkey(key.into()),
@@ -662,11 +789,11 @@ impl<'a> MarshalInto for TSK<'a> {
     }
 
     fn serialize_into(&self, buf: &mut [u8]) -> Result<usize> {
-        generic_serialize_into(self, buf)
+        generic_serialize_into(self, self.serialized_len(), buf)
     }
 
     fn export_into(&self, buf: &mut [u8]) -> Result<usize> {
-        generic_export_into(self, buf)
+        generic_export_into(self, self.serialized_len(), buf)
     }
 }
 
@@ -771,7 +898,7 @@ mod test {
             &mut keypair, &cert,
             signature::SignatureBuilder::new(SignatureType::SubkeyBinding)
                 .set_key_flags(
-                    &KeyFlags::default().set_transport_encryption(true))
+                    &KeyFlags::empty().set_transport_encryption())
                 .unwrap()
                 .set_exportable_certification(false).unwrap()).unwrap();
 
@@ -782,6 +909,7 @@ mod test {
                 cert.primary_key().with_policy(p, None).unwrap()
                     .direct_key_signature().unwrap().clone())
                     .set_type(SignatureType::PositiveCertification)
+                    .preserve_signature_creation_time().unwrap()
                     .set_exportable_certification(false).unwrap()).unwrap();
 
         let ua = UserAttribute::new(&[
@@ -793,9 +921,10 @@ mod test {
                 cert.primary_key().with_policy(p, None).unwrap()
                     .direct_key_signature().unwrap().clone())
                 .set_type(SignatureType::PositiveCertification)
+                .preserve_signature_creation_time().unwrap()
                 .set_exportable_certification(false).unwrap()).unwrap();
 
-        let cert = cert.merge_packets(vec![
+        let cert = cert.insert_packets(vec![
             Packet::SecretSubkey(key), key_binding.into(),
             uid.into(), uid_binding.into(),
             ua.into(), ua_binding.into(),
