@@ -1,6 +1,5 @@
 //! Storage backend.
 
-use failure;
 use std::cmp;
 use std::fmt;
 use std::io;
@@ -230,7 +229,7 @@ impl node::Server for NodeServer {
                              -> Promise<(), capnp::Error> {
         bind_results!(results);
         let fingerprint = pry!(pry!(params.get()).get_fingerprint());
-        let fingerprint = sry!(Fingerprint::from_hex(fingerprint));
+        let fingerprint: openpgp::Fingerprint = sry!(fingerprint.parse());
         let key_id = sry!(KeyServer::lookup(&self.c, &fingerprint));
 
         pry!(pry!(results.get().get_result()).set_ok(
@@ -336,7 +335,7 @@ impl node::mapping::Server for MappingServer {
         bind_results!(results);
         let params = pry!(params.get());
         let fp = pry!(params.get_fingerprint());
-        let fp = sry!(Fingerprint::from_hex(fp)
+        let fp = sry!(fp.parse()
                       .map_err(|_| node::Error::MalformedFingerprint));
         let label = pry!(params.get_label());
 
@@ -569,7 +568,7 @@ impl node::binding::Server for BindingServer {
         // If we found one, convert it to Cert.
         let current = if let Some(current) = key {
             let current = sry!(Cert::from_bytes(&current));
-            if current.fingerprint().to_hex() != fingerprint {
+            if format!("{:X}", current.fingerprint()) != fingerprint {
                 // Inconsistent database.
                 fail!(node::Error::SystemError);
             }
@@ -579,7 +578,7 @@ impl node::binding::Server for BindingServer {
         };
 
         // Check for conflicts.
-        if new.fingerprint().to_hex() != fingerprint {
+        if format!("{:X}", new.fingerprint()) != fingerprint {
             if force {
                 // Update binding, and retry.
                 let key_id =
@@ -717,7 +716,7 @@ impl KeyServer {
     ///
     /// On success, the id of the key is returned.
     fn lookup(c: &Connection, fp: &Fingerprint) -> Result<ID> {
-        let fp = fp.to_hex();
+        let fp = format!("{:X}", fp);
         Ok(c.query_row(
             "SELECT id FROM keys WHERE fingerprint = ?1",
             &[&fp], |row| row.get(0))?)
@@ -727,7 +726,7 @@ impl KeyServer {
     ///
     /// On success, the id of the key is returned.
     fn lookup_by_id(c: &Connection, keyid: &KeyID) -> Result<ID> {
-        let keyid = format!("%{}", keyid.to_hex());
+        let keyid = format!("%{:X}", keyid);
         Ok(c.query_row(
             "SELECT id FROM keys WHERE fingerprint LIKE ?1",
             &[&keyid], |row| row.get(0))?)
@@ -737,7 +736,7 @@ impl KeyServer {
     ///
     /// On success, the id of the key is returned.
     fn lookup_or_create(c: &Connection, fp: &Fingerprint) -> Result<ID> {
-        let fp = fp.to_hex();
+        let fp = format!("{:X}", fp);
         if let Ok(x) = c.query_row(
             "SELECT id FROM keys WHERE fingerprint = ?1",
             &[&fp], |row| row.get(0)) {
@@ -781,7 +780,7 @@ impl KeyServer {
         if let Some(current) = key {
             let current = Cert::from_bytes(&current)?;
 
-            if current.fingerprint().to_hex() != fingerprint {
+            if format!("{:X}", current.fingerprint()) != fingerprint {
                 // Inconsistent database.
                 return Err(node::Error::SystemError.into());
             }
@@ -905,7 +904,7 @@ impl KeyServer {
                  ORDER BY keys.update_at LIMIT 1",
             &[&network_policy_u8 as &dyn ToSql, &Timestamp::now()],
             |row| Ok((row.get(0)?, row.get(1)?)))?;
-        let fingerprint = openpgp::Fingerprint::from_hex(&fingerprint)
+        let fingerprint = fingerprint.parse::<openpgp::Fingerprint>()
             .map_err(|_| node::Error::SystemError)?;
 
         let ctx = core::Context::configure()
@@ -920,7 +919,7 @@ impl KeyServer {
     /// Updates the key that was least recently updated.
     fn update(c: &Rc<Connection>,
               network_policy: core::NetworkPolicy)
-              -> Box<dyn Future<Item=Duration, Error=failure::Error> + 'static> {
+              -> Box<dyn Future<Item=Duration, Error=anyhow::Error> + 'static> {
         let (key, id, mut keyserver)
             = match Self::update_helper(c, network_policy) {
             Ok((key, id, keyserver)) => (key, id, keyserver),
@@ -1004,7 +1003,7 @@ impl Query for KeyServer {
             "SELECT fingerprint FROM keys WHERE id = ?1",
             &[&self.id], |row| -> rusqlite::Result<String> { row.get(0) })
             .ok()
-            .and_then(|fp| Fingerprint::from_hex(&fp).ok())
+            .and_then(|fp| fp.parse::<openpgp::Fingerprint>().ok())
             .map(|fp| KeyID::from(fp).to_string())
             .unwrap_or(
                 format!("{}::{}", Self::table_name(), self.id())
@@ -1277,8 +1276,8 @@ impl From<rusqlite::Error> for node::Error {
     }
 }
 
-impl From<failure::Error> for node::Error {
-    fn from(e: failure::Error) -> Self {
+impl From<anyhow::Error> for node::Error {
+    fn from(e: anyhow::Error) -> Self {
         if let Some(e) = e.downcast_ref::<openpgp::Error>() {
             return match e {
                 &openpgp::Error::MalformedCert(_) =>
